@@ -16,6 +16,7 @@
 
 #if __ARM_NEON
 #include <arm_neon.h>
+#include "arm_usability.h"
 #endif // __ARM_NEON
 
 namespace ncnn {
@@ -265,7 +266,12 @@ int PReLU_arm::forward_inplace_fp16sa(Mat& bottom_top_blob, const Option& opt) c
             }
             else
             {
+#if defined(_MSC_VER) && !defined(__clang__)
+                float16x4_t _slope0 = vcvt_f16_f32(vdupq_n_f32(slope_data[0]));
+                float16x8_t _slope = vcombine_f16(_slope0, _slope0);
+#else
                 float16x8_t _slope = vdupq_n_f16((__fp16)slope_data[0]);
+#endif
 
                 #pragma omp parallel for num_threads(opt.num_threads)
                 for (int i = 0; i < w; i++)
@@ -336,10 +342,14 @@ int PReLU_arm::forward_inplace_fp16sa(Mat& bottom_top_blob, const Option& opt) c
 
     if (elempack == 4)
     {
-        float16x4_t _zero = vdup_n_f16(0.f);
-
         if (dims == 1)
         {
+#if defined(_MSC_VER) && !defined(__clang__)
+            float16x8_t _zero = vdupq_n_f16(0.f);
+#else
+            float16x4_t _zero = vdup_n_f16(0.f);
+#endif
+
             int w = bottom_top_blob.w;
 
             if (num_slope > 1)
@@ -353,7 +363,11 @@ int PReLU_arm::forward_inplace_fp16sa(Mat& bottom_top_blob, const Option& opt) c
 
                     float16x4_t _p = vld1_f16(ptr);
                     float16x4_t _slope = vcvt_f16_f32(vld1q_f32(slope + i * 4));
+#if defined(_MSC_VER) && !defined(__clang__)
+                    uint16x4_t _lemask = vcle_f16(_p, vget_low_f16(_zero));
+#else
                     uint16x4_t _lemask = vcle_f16(_p, _zero);
+#endif
                     float16x4_t _ps = vmul_f16(_p, _slope);
                     _p = vbsl_f16(_lemask, _ps, _p);
                     vst1_f16(ptr, _p);
@@ -361,7 +375,11 @@ int PReLU_arm::forward_inplace_fp16sa(Mat& bottom_top_blob, const Option& opt) c
             }
             else
             {
+#if defined(_MSC_VER) && !defined(__clang__)
+                float16x8_t _slope = vdupq_n_f16((__fp16)slope_data[0]);
+#else
                 float16x4_t _slope = vdup_n_f16((__fp16)slope_data[0]);
+#endif
 
                 #pragma omp parallel for num_threads(opt.num_threads)
                 for (int i = 0; i < w; i++)
@@ -369,8 +387,13 @@ int PReLU_arm::forward_inplace_fp16sa(Mat& bottom_top_blob, const Option& opt) c
                     __fp16* ptr = (__fp16*)bottom_top_blob + i * 4;
 
                     float16x4_t _p = vld1_f16(ptr);
+#if defined(_MSC_VER) && !defined(__clang__)
+                    uint16x4_t _lemask = vcle_f16(_p, vget_low_f16(_zero));
+                    float16x4_t _ps = vmul_f16(_p, vget_low_f16(_slope));
+#else
                     uint16x4_t _lemask = vcle_f16(_p, _zero);
                     float16x4_t _ps = vmul_f16(_p, _slope);
+#endif
                     _p = vbsl_f16(_lemask, _ps, _p);
                     vst1_f16(ptr, _p);
                 }
@@ -386,6 +409,7 @@ int PReLU_arm::forward_inplace_fp16sa(Mat& bottom_top_blob, const Option& opt) c
             for (int i = 0; i < h; i++)
             {
                 __fp16* ptr = bottom_top_blob.row<__fp16>(i);
+                float16x4_t _zero = vdup_n_f16(0.f);
                 float16x4_t _slope = num_slope > 1 ? vcvt_f16_f32(vld1q_f32((const float*)slope_data + i * 4)) : vdup_n_f16((__fp16)slope_data[0]);
 
                 for (int j = 0; j < w; j++)
@@ -412,6 +436,7 @@ int PReLU_arm::forward_inplace_fp16sa(Mat& bottom_top_blob, const Option& opt) c
             for (int q = 0; q < channels; q++)
             {
                 __fp16* ptr = bottom_top_blob.channel(q);
+                float16x4_t _zero = vdup_n_f16(0.f);
                 float16x4_t _slope = num_slope > 1 ? vcvt_f16_f32(vld1q_f32((const float*)slope_data + q * 4)) : vdup_n_f16((__fp16)slope_data[0]);
 
                 for (int i = 0; i < size; i++)
@@ -475,7 +500,11 @@ int PReLU_arm::forward_inplace_fp16sa(Mat& bottom_top_blob, const Option& opt) c
             const float slope = num_slope > 1 ? slope_data[i] : slope_data[0];
 
             float16x4_t _zero = vdup_n_f16(0.f);
+#if defined(_MSC_VER) && !defined(__clang__)
+            float16x4_t _slope = vcvt_f16_f32(vdupq_n_f32(slope));
+#else
             float16x4_t _slope = vdup_n_f16((__fp16)slope);
+#endif
 
             int j = 0;
             for (; j + 3 < w; j += 4)
@@ -514,7 +543,11 @@ int PReLU_arm::forward_inplace_fp16sa(Mat& bottom_top_blob, const Option& opt) c
             const float slope = num_slope > 1 ? slope_data[q] : slope_data[0];
 
             float16x4_t _zero = vdup_n_f16(0.f);
+#if defined(_MSC_VER) && !defined(__clang__)
+            float16x4_t _slope = vcvt_f16_f32(vdupq_n_f32(slope));
+#else
             float16x4_t _slope = vdup_n_f16((__fp16)slope);
+#endif
 
             int j = 0;
             for (; j + 3 < size; j += 4)
