@@ -84,9 +84,7 @@ int ConvolutionDepthWise_loongarch::create_pipeline(const Option& opt)
         }
 
         if (opt.lightmode)
-        {
             weight_data.release();
-        }
 
         return 0;
     }
@@ -95,9 +93,7 @@ int ConvolutionDepthWise_loongarch::create_pipeline(const Option& opt)
     create_group_ops(opt);
 
     if (opt.lightmode)
-    {
         weight_data.release();
-    }
 
     return 0;
 }
@@ -125,7 +121,7 @@ int ConvolutionDepthWise_loongarch::create_group_ops(const Option& opt)
         if (bias_term)
             bias_data_g = bias_data.range(num_output_g * g, num_output_g);
 
-        ncnn::Layer* op = ncnn::create_layer(ncnn::LayerType::Convolution);
+        ncnn::Layer* op = ncnn::create_layer_cpu(ncnn::LayerType::Convolution);
 
         // set param
         ncnn::ParamDict pd;
@@ -537,7 +533,7 @@ int ConvolutionDepthWise_loongarch::forward(const std::vector<Mat>& bottom_blobs
         bias_data_flattened.elempack = 1;
     }
 
-    ncnn::Layer* op = ncnn::create_layer(ncnn::LayerType::ConvolutionDepthWise);
+    ncnn::Layer* op = ncnn::create_layer_cpu(ncnn::LayerType::ConvolutionDepthWise);
 
     ncnn::ParamDict pd;
     pd.set(0, _num_output);
@@ -606,6 +602,9 @@ int ConvolutionDepthWise_loongarch::create_pipeline_int8_loongarch(const Option&
             weight_data_tm = weight_data;
         }
 
+        if (opt.lightmode)
+            weight_data.release();
+
         return 0;
     }
 
@@ -613,9 +612,7 @@ int ConvolutionDepthWise_loongarch::create_pipeline_int8_loongarch(const Option&
     create_group_ops(opt);
 
     if (opt.lightmode)
-    {
         weight_data.release();
-    }
 
     return 0;
 }
