@@ -20,11 +20,9 @@ namespace ncnn {
 Noop::Noop()
 {
     support_inplace = true;
-    support_vulkan = true;
     support_packing = true;
     support_fp16_storage = cpu_support_arm_asimdhp() || cpu_support_riscv_zfh();
     support_bf16_storage = true;
-    support_image_storage = true;
 }
 
 int Noop::forward_inplace(std::vector<Mat>& /*bottom_top_blobs*/, const Option& /*opt*/) const
@@ -32,16 +30,4 @@ int Noop::forward_inplace(std::vector<Mat>& /*bottom_top_blobs*/, const Option&
     return 0;
 }
 
-#if NCNN_VULKAN
-int Noop::forward_inplace(std::vector<VkMat>& /*bottom_top_blobs*/, VkCompute& /*cmd*/, const Option& /*opt*/) const
-{
-    return 0;
-}
-
-int Noop::forward_inplace(std::vector<VkImageMat>& /*bottom_top_blobs*/, VkCompute& /*cmd*/, const Option& /*opt*/) const
-{
-    return 0;
-}
-#endif // NCNN_VULKAN
-
 } // namespace ncnn
