@@ -19,7 +19,7 @@
 
 namespace ncnn {
 
-class Padding_riscv : virtual public Padding
+class Padding_riscv : public Padding
 {
 public:
     Padding_riscv();
