@@ -20,8 +20,6 @@
 #include "rvv_mathfun_fp16s.h"
 #endif // __riscv_vector
 
-#include <math.h>
-
 namespace ncnn {
 
 UnaryOp_riscv::UnaryOp_riscv()
@@ -127,9 +125,13 @@ struct unary_op_rsqrt
 {
     vfloat32m8_t operator()(const vfloat32m8_t& x, const size_t& vl) const
     {
+#if C906
+        vfloat32m8_t _reciprocal = vfrdiv_vf_f32m8(vfsqrt_v_f32m8(x, vl), 1.f, vl);
+#else
         vfloat32m8_t _reciprocal = vfrsqrt7_v_f32m8(x, vl);
         _reciprocal = vfmul_vv_f32m8(vfrsub_vf_f32m8(vfmul_vv_f32m8(vfmul_vf_f32m8(x, 0.5f, vl), vfmul_vv_f32m8(_reciprocal, _reciprocal, vl), vl), 1.5f, vl), _reciprocal, vl);
         // _reciprocal = vfmul_vv_f32m8(vfrsub_vf_f32m8(vfmul_vv_f32m8(vfmul_vf_f32m8(x, 0.5f, vl), vfmul_vv_f32m8(_reciprocal, _reciprocal, vl), vl), 1.5f, vl), _reciprocal, vl);
+#endif
         return _reciprocal;
     }
 };
@@ -175,7 +177,7 @@ struct unary_op_tan
         vse32_v_f32m8(tmp.data(), x, vl);
         for (size_t i = 0; i < vl; i++)
         {
-            tmp[i] = tan(tmp[i]);
+            tmp[i] = tanf(tmp[i]);
         }
         return vle32_v_f32m8(tmp.data(), vl);
     }
@@ -190,7 +192,7 @@ struct unary_op_asin
         vse32_v_f32m8(tmp.data(), x, vl);
         for (size_t i = 0; i < vl; i++)
         {
-            tmp[i] = asin(tmp[i]);
+            tmp[i] = asinf(tmp[i]);
         }
         return vle32_v_f32m8(tmp.data(), vl);
     }
@@ -205,7 +207,7 @@ struct unary_op_acos
         vse32_v_f32m8(tmp.data(), x, vl);
         for (size_t i = 0; i < vl; i++)
         {
-            tmp[i] = acos(tmp[i]);
+            tmp[i] = acosf(tmp[i]);
         }
         return vle32_v_f32m8(tmp.data(), vl);
     }
@@ -220,7 +222,7 @@ struct unary_op_atan
         vse32_v_f32m8(tmp.data(), x, vl);
         for (size_t i = 0; i < vl; i++)
         {
-            tmp[i] = atan(tmp[i]);
+            tmp[i] = atanf(tmp[i]);
         }
         return vle32_v_f32m8(tmp.data(), vl);
     }
@@ -230,9 +232,13 @@ struct unary_op_reciprocal
 {
     vfloat32m8_t operator()(const vfloat32m8_t& x, const size_t& vl) const
     {
+#if C906
+        vfloat32m8_t _reciprocal = vfrdiv_vf_f32m8(x, 1.f, vl);
+#else
         vfloat32m8_t _reciprocal = vfrec7_v_f32m8(x, vl);
         _reciprocal = vfmul_vv_f32m8(vfrsub_vf_f32m8(vfmul_vv_f32m8(x, _reciprocal, vl), 2.f, vl), _reciprocal, vl);
         // _reciprocal = vfmul_vv_f32m8(vfrsub_vf_f32m8(vfmul_vv_f32m8(x, _reciprocal, vl), 2.f, vl), _reciprocal, vl);
+#endif
         return _reciprocal;
     }
 };
@@ -253,6 +259,38 @@ struct unary_op_log10
     }
 };
 
+struct unary_op_round
+{
+    vfloat32m8_t operator()(const vfloat32m8_t& x, const size_t& vl) const
+    {
+        return vfcvt_f_x_v_f32m8(vfcvt_x_f_v_i32m8(x, vl), vl);
+    }
+};
+
+struct unary_op_trunc
+{
+    vfloat32m8_t operator()(const vfloat32m8_t& x, const size_t& vl) const
+    {
+#if C906
+        // simulate trunc with floor positives and ceil negative
+        // xi = round(x)
+        // floorx = xi - (xi > x)
+        // ceilx = xi + (xi < x)
+        // truncx = x >= 0 ? floorx : ceilx
+        vint32m8_t _xi = vfcvt_x_f_v_i32m8(x, vl);
+        vfloat32m8_t _xf = vfcvt_f_x_v_f32m8(_xi, vl);
+        vbool4_t _floormask = vmfgt_vv_f32m8_b4(_xf, x, vl);
+        vint32m8_t _floorx = vsub_vx_i32m8_m(_floormask, _xi, _xi, 1, vl);
+        vbool4_t _ceilmask = vmflt_vv_f32m8_b4(_xf, x, vl);
+        vint32m8_t _ceilx = vadd_vx_i32m8_m(_ceilmask, _xi, _xi, 1, vl);
+        vbool4_t _negative = vmflt_vf_f32m8_b4(x, 0.f, vl);
+        return vfcvt_f_x_v_f32m8(vmerge_vvm_i32m8(_negative, _floorx, _ceilx, vl), vl);
+#else
+        return vfcvt_f_x_v_f32m8(vfcvt_rtz_x_f_v_i32m8(x, vl), vl);
+#endif
+    }
+};
+
 } // namespace UnaryOp_riscv_functor
 #endif // __riscv_vector
 
@@ -322,6 +360,12 @@ int UnaryOp_riscv::forward_inplace(Mat& bottom_top_blob, const Option& opt) cons
     if (op_type == Operation_LOG10)
         return unary_op_inplace<unary_op_log10>(bottom_top_blob, opt);
 
+    if (op_type == Operation_ROUND)
+        return unary_op_inplace<unary_op_round>(bottom_top_blob, opt);
+
+    if (op_type == Operation_TRUNC)
+        return unary_op_inplace<unary_op_trunc>(bottom_top_blob, opt);
+
     return 0;
 #else  // __riscv_vector
     return UnaryOp::forward_inplace(bottom_top_blob, opt);
@@ -421,9 +465,13 @@ struct unary_op_rsqrt_fp16s
 {
     vfloat16m8_t operator()(const vfloat16m8_t& x, const size_t& vl) const
     {
+#if C906
+        vfloat16m8_t _reciprocal = vfrdiv_vf_f16m8(vfsqrt_v_f16m8(x, vl), 1.f, vl);
+#else
         vfloat16m8_t _reciprocal = vfrsqrt7_v_f16m8(x, vl);
         _reciprocal = vfmul_vv_f16m8(vfrsub_vf_f16m8(vfmul_vv_f16m8(vfmul_vf_f16m8(x, 0.5f, vl), vfmul_vv_f16m8(_reciprocal, _reciprocal, vl), vl), 1.5f, vl), _reciprocal, vl);
         // _reciprocal = vfmul_vv_f16m8(vfrsub_vf_f16m8(vfmul_vv_f16m8(vfmul_vf_f16m8(x, 0.5f, vl), vfmul_vv_f16m8(_reciprocal, _reciprocal, vl), vl), 1.5f, vl), _reciprocal, vl);
+#endif
         return _reciprocal;
     }
 };
@@ -469,7 +517,7 @@ struct unary_op_tan_fp16s
         vse16_v_f16m8(tmp.data(), x, vl);
         for (size_t i = 0; i < vl; i++)
         {
-            tmp[i] = tan((float)tmp[i]);
+            tmp[i] = tanf((float)tmp[i]);
         }
         return vle16_v_f16m8(tmp.data(), vl);
     }
@@ -484,7 +532,7 @@ struct unary_op_asin_fp16s
         vse16_v_f16m8(tmp.data(), x, vl);
         for (size_t i = 0; i < vl; i++)
         {
-            tmp[i] = asin((float)tmp[i]);
+            tmp[i] = asinf((float)tmp[i]);
         }
         return vle16_v_f16m8(tmp.data(), vl);
     }
@@ -499,7 +547,7 @@ struct unary_op_acos_fp16s
         vse16_v_f16m8(tmp.data(), x, vl);
         for (size_t i = 0; i < vl; i++)
         {
-            tmp[i] = acos((float)tmp[i]);
+            tmp[i] = acosf((float)tmp[i]);
         }
         return vle16_v_f16m8(tmp.data(), vl);
     }
@@ -514,7 +562,7 @@ struct unary_op_atan_fp16s
         vse16_v_f16m8(tmp.data(), x, vl);
         for (size_t i = 0; i < vl; i++)
         {
-            tmp[i] = atan((float)tmp[i]);
+            tmp[i] = atanf((float)tmp[i]);
         }
         return vle16_v_f16m8(tmp.data(), vl);
     }
@@ -524,9 +572,13 @@ struct unary_op_reciprocal_fp16s
 {
     vfloat16m8_t operator()(const vfloat16m8_t& x, const size_t& vl) const
     {
+#if C906
+        vfloat16m8_t _reciprocal = vfrdiv_vf_f16m8(x, 1.f, vl);
+#else
         vfloat16m8_t _reciprocal = vfrec7_v_f16m8(x, vl);
         _reciprocal = vfmul_vv_f16m8(vfrsub_vf_f16m8(vfmul_vv_f16m8(x, _reciprocal, vl), 2.f, vl), _reciprocal, vl);
         // _reciprocal = vfmul_vv_f16m8(vfrsub_vf_f16m8(vfmul_vv_f16m8(x, _reciprocal, vl), 2.f, vl), _reciprocal, vl);
+#endif
         return _reciprocal;
     }
 };
@@ -547,6 +599,38 @@ struct unary_op_log10_fp16s
     }
 };
 
+struct unary_op_round_fp16s
+{
+    vfloat16m8_t operator()(const vfloat16m8_t& x, const size_t& vl) const
+    {
+        return vfcvt_f_x_v_f16m8(vfcvt_x_f_v_i16m8(x, vl), vl);
+    }
+};
+
+struct unary_op_trunc_fp16s
+{
+    vfloat16m8_t operator()(const vfloat16m8_t& x, const size_t& vl) const
+    {
+#if C906
+        // simulate trunc with floor positives and ceil negative
+        // xi = round(x)
+        // floorx = xi - (xi > x)
+        // ceilx = xi + (xi < x)
+        // truncx = x >= 0 ? floorx : ceilx
+        vint16m8_t _xi = vfcvt_x_f_v_i16m8(x, vl);
+        vfloat16m8_t _xf = vfcvt_f_x_v_f16m8(_xi, vl);
+        vbool2_t _floormask = vmfgt_vv_f16m8_b2(_xf, x, vl);
+        vint16m8_t _floorx = vsub_vx_i16m8_m(_floormask, _xi, _xi, 1, vl);
+        vbool2_t _ceilmask = vmflt_vv_f16m8_b2(_xf, x, vl);
+        vint16m8_t _ceilx = vadd_vx_i16m8_m(_ceilmask, _xi, _xi, 1, vl);
+        vbool2_t _negative = vmflt_vf_f16m8_b2(x, 0.f, vl);
+        return vfcvt_f_x_v_f16m8(vmerge_vvm_i16m8(_negative, _floorx, _ceilx, vl), vl);
+#else
+        return vfcvt_f_x_v_f16m8(vfcvt_rtz_x_f_v_i16m8(x, vl), vl);
+#endif
+    }
+};
+
 } // namespace UnaryOp_riscv_functor
 
 int UnaryOp_riscv::forward_inplace_fp16s(Mat& bottom_top_blob, const Option& opt) const
@@ -607,6 +691,12 @@ int UnaryOp_riscv::forward_inplace_fp16s(Mat& bottom_top_blob, const Option& opt
     if (op_type == Operation_LOG10)
         return unary_op_inplace_fp16s<unary_op_log10_fp16s>(bottom_top_blob, opt);
 
+    if (op_type == Operation_ROUND)
+        return unary_op_inplace_fp16s<unary_op_round_fp16s>(bottom_top_blob, opt);
+
+    if (op_type == Operation_TRUNC)
+        return unary_op_inplace_fp16s<unary_op_trunc_fp16s>(bottom_top_blob, opt);
+
     return 0;
 }
 #endif // __riscv_vector && __riscv_zfh
