@@ -14,7 +14,8 @@
 
 #include "unaryop_x86.h"
 
-#include <math.h>
+// #include <fenv.h>
+#include <float.h>
 
 #if __SSE2__
 #include <emmintrin.h>
@@ -41,6 +42,7 @@ UnaryOp_x86::UnaryOp_x86()
     support_packing = true;
 #endif // __SSE2__
 }
+
 template<typename Op>
 static int unary_op_inplace(Mat& a, const Option& opt)
 {
@@ -99,24 +101,24 @@ static int unary_op_inplace(Mat& a, const Option& opt)
 namespace UnaryOp_x86_functor {
 struct unary_op_abs
 {
-    float func(const float& x) const
+    NCNN_FORCEINLINE float func(const float& x) const
     {
-        return (float)fabs(x);
+        return (float)fabsf(x);
     }
 #if __SSE2__
-    __m128 func_pack4(const __m128& x) const
+    NCNN_FORCEINLINE __m128 func_pack4(const __m128& x) const
     {
-        return abs_sse(x);
+        return abs_ps(x);
     }
 #if __AVX__
-    __m256 func_pack8(const __m256& x) const
+    NCNN_FORCEINLINE __m256 func_pack8(const __m256& x) const
     {
-        return abs_avx(x);
+        return abs256_ps(x);
     }
 #if __AVX512F__
-    __m512 func_pack16(const __m512& x) const
+    NCNN_FORCEINLINE __m512 func_pack16(const __m512& x) const
     {
-        return abs_avx512(x);
+        return abs512_ps(x);
     }
 #endif // __AVX512F__
 #endif // __AVX__
@@ -125,22 +127,22 @@ struct unary_op_abs
 
 struct unary_op_neg
 {
-    float func(const float& x) const
+    NCNN_FORCEINLINE float func(const float& x) const
     {
         return -x;
     }
 #if __SSE2__
-    __m128 func_pack4(const __m128& x) const
+    NCNN_FORCEINLINE __m128 func_pack4(const __m128& x) const
     {
         return _mm_sub_ps(_mm_setzero_ps(), x);
     }
 #if __AVX__
-    __m256 func_pack8(const __m256& x) const
+    NCNN_FORCEINLINE __m256 func_pack8(const __m256& x) const
     {
         return _mm256_sub_ps(_mm256_setzero_ps(), x);
     }
 #if __AVX512F__
-    __m512 func_pack16(const __m512& x) const
+    NCNN_FORCEINLINE __m512 func_pack16(const __m512& x) const
     {
         return _mm512_sub_ps(_mm512_setzero_ps(), x);
     }
@@ -151,58 +153,22 @@ struct unary_op_neg
 
 struct unary_op_floor
 {
-    float func(const float& x) const
+    NCNN_FORCEINLINE float func(const float& x) const
     {
-        return (float)floor(x);
+        return (float)floorf(x);
     }
 #if __SSE2__
-    __m128 func_pack4(const __m128& x) const
+    NCNN_FORCEINLINE __m128 func_pack4(const __m128& x) const
     {
-#if __SSE4_1__
-        return _mm_floor_ps(x);
-#endif // __SSE4_1__
-
-        // Use negative zero as the sign bit mask.
-        const __m128 magic_negative_zero = _mm_set_ps1(-0.0f);
-
-        // The smallest float number that have no fractional part. (2^23)
-        const __m128 magic_smallest_no_fraction = _mm_set_ps1(8388608.0f);
-
-        // absolute = abs(x);
-        __m128 absolute = _mm_andnot_ps(magic_negative_zero, x);
-
-        // negative_mask = magic_negative_zero && x;
-        __m128 negative_mask = _mm_and_ps(magic_negative_zero, x);
-
-        // no_fraction = (magic_smallest_no_fraction < absolute);
-        __m128 no_fraction = _mm_cmplt_ps(magic_smallest_no_fraction, absolute);
-
-        // truncated = static_cast<float>(static_cast<uint32_t>(absolute));
-        __m128 truncated = _mm_cvtepi32_ps(_mm_cvttps_epi32(absolute));
-
-        // truncated_with_sign = (truncated || negative_mask);
-        __m128 truncated_with_sign = _mm_or_ps(truncated, negative_mask);
-
-        // negative_fix = ((x < truncated_with_sign) ? 1.0f : 0.0f);
-        __m128 negative_fix = _mm_and_ps(
-                                  _mm_cmplt_ps(x, truncated_with_sign),
-                                  _mm_set_ps1(1.0f));
-
-        // fixed_result = truncated_with_sign - negative_fix;
-        __m128 fixed_result = _mm_sub_ps(truncated_with_sign, negative_fix);
-
-        // return ((x && no_fraction) || (!no_fraction && fixed_result));
-        return _mm_or_ps(
-                   _mm_and_ps(x, no_fraction),
-                   _mm_andnot_ps(no_fraction, fixed_result));
+        return floor_ps(x);
     }
 #if __AVX__
-    __m256 func_pack8(const __m256& x) const
+    NCNN_FORCEINLINE __m256 func_pack8(const __m256& x) const
     {
         return _mm256_floor_ps(x);
     }
 #if __AVX512F__
-    __m512 func_pack16(const __m512& x) const
+    NCNN_FORCEINLINE __m512 func_pack16(const __m512& x) const
     {
         return _mm512_roundscale_ps(x, _MM_FROUND_TO_NEG_INF);
     }
@@ -213,60 +179,22 @@ struct unary_op_floor
 
 struct unary_op_ceil
 {
-    float func(const float& x) const
+    NCNN_FORCEINLINE float func(const float& x) const
     {
-        return (float)ceil(x);
+        return (float)ceilf(x);
     }
 #if __SSE2__
-    __m128 func_pack4(const __m128& x) const
+    NCNN_FORCEINLINE __m128 func_pack4(const __m128& x) const
     {
-#if __SSE4_1__
-        return _mm_ceil_ps(x);
-#endif // __SSE4_1__
-
-        // Use negative zero as the sign bit mask.
-        const __m128 magic_negative_zero = _mm_set_ps1(-0.0f);
-
-        // The smallest float number that have no fractional part. (2^23)
-        const __m128 magic_smallest_no_fraction = _mm_set_ps1(8388608.0f);
-
-        // absolute = abs(x);
-        __m128 absolute = _mm_andnot_ps(magic_negative_zero, x);
-
-        // negative_mask = magic_negative_zero && x;
-        __m128 negative_mask = _mm_and_ps(magic_negative_zero, x);
-
-        // no_fraction = (magic_smallest_no_fraction < absolute);
-        __m128 no_fraction = _mm_cmplt_ps(magic_smallest_no_fraction, absolute);
-
-        // truncated = static_cast<float>(static_cast<uint32_t>(absolute));
-        __m128 truncated = _mm_cvtepi32_ps(_mm_cvttps_epi32(absolute));
-
-        // truncated_with_sign = (truncated || negative_mask);
-        __m128 truncated_with_sign = _mm_or_ps(truncated, negative_mask);
-
-        // positive_fix = ((x > -0.0f) && (x > truncated_with_sign) ? -1.0f : 0.0f);
-        __m128 positive_fix = _mm_and_ps(
-                                  _mm_and_ps(
-                                      _mm_cmpgt_ps(x, magic_negative_zero),
-                                      _mm_cmpgt_ps(x, truncated_with_sign)),
-                                  _mm_set_ps1(-1.0f));
-
-        // fixed_result = truncated_with_sign - positive_fix;
-        __m128 fixed_result = _mm_sub_ps(truncated_with_sign, positive_fix);
-
-        // return ((x && no_fraction) || (!no_fraction && fixed_result));
-        return _mm_or_ps(
-                   _mm_and_ps(x, no_fraction),
-                   _mm_andnot_ps(no_fraction, fixed_result));
+        return ceil_ps(x);
     }
 #if __AVX__
-    __m256 func_pack8(const __m256& x) const
+    NCNN_FORCEINLINE __m256 func_pack8(const __m256& x) const
     {
         return _mm256_ceil_ps(x);
     }
 #if __AVX512F__
-    __m512 func_pack16(const __m512& x) const
+    NCNN_FORCEINLINE __m512 func_pack16(const __m512& x) const
     {
         return _mm512_roundscale_ps(x, _MM_FROUND_TO_POS_INF);
     }
@@ -277,22 +205,22 @@ struct unary_op_ceil
 
 struct unary_op_square
 {
-    float func(const float& x) const
+    NCNN_FORCEINLINE float func(const float& x) const
     {
         return x * x;
     }
 #if __SSE2__
-    __m128 func_pack4(const __m128& x) const
+    NCNN_FORCEINLINE __m128 func_pack4(const __m128& x) const
     {
         return _mm_mul_ps(x, x);
     }
 #if __AVX__
-    __m256 func_pack8(const __m256& x) const
+    NCNN_FORCEINLINE __m256 func_pack8(const __m256& x) const
     {
         return _mm256_mul_ps(x, x);
     }
 #if __AVX512F__
-    __m512 func_pack16(const __m512& x) const
+    NCNN_FORCEINLINE __m512 func_pack16(const __m512& x) const
     {
         return _mm512_mul_ps(x, x);
     }
@@ -303,22 +231,22 @@ struct unary_op_square
 
 struct unary_op_sqrt
 {
-    float func(const float& x) const
+    NCNN_FORCEINLINE float func(const float& x) const
     {
-        return (float)sqrt(x);
+        return (float)sqrtf(x);
     }
 #if __SSE2__
-    __m128 func_pack4(const __m128& x) const
+    NCNN_FORCEINLINE __m128 func_pack4(const __m128& x) const
     {
         return _mm_sqrt_ps(x);
     }
 #if __AVX__
-    __m256 func_pack8(const __m256& x) const
+    NCNN_FORCEINLINE __m256 func_pack8(const __m256& x) const
     {
         return _mm256_sqrt_ps(x);
     }
 #if __AVX512F__
-    __m512 func_pack16(const __m512& x) const
+    NCNN_FORCEINLINE __m512 func_pack16(const __m512& x) const
     {
         return _mm512_sqrt_ps(x);
     }
@@ -329,22 +257,22 @@ struct unary_op_sqrt
 
 struct unary_op_rsqrt
 {
-    float func(const float& x) const
+    NCNN_FORCEINLINE float func(const float& x) const
     {
-        return (float)(1.f / sqrt(x));
+        return 1.f / sqrtf(x);
     }
 #if __SSE2__
-    __m128 func_pack4(const __m128& x) const
+    NCNN_FORCEINLINE __m128 func_pack4(const __m128& x) const
     {
         return _mm_rsqrt_ps(x);
     }
 #if __AVX__
-    __m256 func_pack8(const __m256& x) const
+    NCNN_FORCEINLINE __m256 func_pack8(const __m256& x) const
     {
         return _mm256_rsqrt_ps(x);
     }
 #if __AVX512F__
-    __m512 func_pack16(const __m512& x) const
+    NCNN_FORCEINLINE __m512 func_pack16(const __m512& x) const
     {
         __m256 _x0 = _mm512_extractf32x8_ps(x, 0);
         __m256 _x1 = _mm512_extractf32x8_ps(x, 1);
@@ -359,22 +287,22 @@ struct unary_op_rsqrt
 
 struct unary_op_exp
 {
-    float func(const float& x) const
+    NCNN_FORCEINLINE float func(const float& x) const
     {
-        return (float)exp(x);
+        return (float)expf(x);
     }
 #if __SSE2__
-    __m128 func_pack4(const __m128& x) const
+    NCNN_FORCEINLINE __m128 func_pack4(const __m128& x) const
     {
         return exp_ps(x);
     }
 #if __AVX__
-    __m256 func_pack8(const __m256& x) const
+    NCNN_FORCEINLINE __m256 func_pack8(const __m256& x) const
     {
         return exp256_ps(x);
     }
 #if __AVX512F__
-    __m512 func_pack16(const __m512& x) const
+    NCNN_FORCEINLINE __m512 func_pack16(const __m512& x) const
     {
         return exp512_ps(x);
     }
@@ -385,22 +313,22 @@ struct unary_op_exp
 
 struct unary_op_log
 {
-    float func(const float& x) const
+    NCNN_FORCEINLINE float func(const float& x) const
     {
-        return (float)log(x);
+        return (float)logf(x);
     }
 #if __SSE2__
-    __m128 func_pack4(const __m128& x) const
+    NCNN_FORCEINLINE __m128 func_pack4(const __m128& x) const
     {
         return log_ps(x);
     }
 #if __AVX__
-    __m256 func_pack8(const __m256& x) const
+    NCNN_FORCEINLINE __m256 func_pack8(const __m256& x) const
     {
         return log256_ps(x);
     }
 #if __AVX512F__
-    __m512 func_pack16(const __m512& x) const
+    NCNN_FORCEINLINE __m512 func_pack16(const __m512& x) const
     {
         return log512_ps(x);
     }
@@ -411,22 +339,22 @@ struct unary_op_log
 
 struct unary_op_sin
 {
-    float func(const float& x) const
+    NCNN_FORCEINLINE float func(const float& x) const
     {
-        return (float)sin(x);
+        return (float)sinf(x);
     }
 #if __SSE2__
-    __m128 func_pack4(const __m128& x) const
+    NCNN_FORCEINLINE __m128 func_pack4(const __m128& x) const
     {
         return sin_ps(x);
     }
 #if __AVX__
-    __m256 func_pack8(const __m256& x) const
+    NCNN_FORCEINLINE __m256 func_pack8(const __m256& x) const
     {
         return sin256_ps(x);
     }
 #if __AVX512F__
-    __m512 func_pack16(const __m512& x) const
+    NCNN_FORCEINLINE __m512 func_pack16(const __m512& x) const
     {
         return sin512_ps(x);
     }
@@ -437,22 +365,22 @@ struct unary_op_sin
 
 struct unary_op_cos
 {
-    float func(const float& x) const
+    NCNN_FORCEINLINE float func(const float& x) const
     {
-        return (float)cos(x);
+        return (float)cosf(x);
     }
 #if __SSE2__
-    __m128 func_pack4(const __m128& x) const
+    NCNN_FORCEINLINE __m128 func_pack4(const __m128& x) const
     {
         return cos_ps(x);
     }
 #if __AVX__
-    __m256 func_pack8(const __m256& x) const
+    NCNN_FORCEINLINE __m256 func_pack8(const __m256& x) const
     {
         return cos256_ps(x);
     }
 #if __AVX512F__
-    __m512 func_pack16(const __m512& x) const
+    NCNN_FORCEINLINE __m512 func_pack16(const __m512& x) const
     {
         return cos512_ps(x);
     }
@@ -463,22 +391,22 @@ struct unary_op_cos
 
 struct unary_op_tan
 {
-    float func(const float& x) const
+    NCNN_FORCEINLINE float func(const float& x) const
     {
-        return (float)tan(x);
+        return (float)tanf(x);
     }
 #if __SSE2__
-    __m128 func_pack4(const __m128& x) const
+    NCNN_FORCEINLINE __m128 func_pack4(const __m128& x) const
     {
         return tan_ps(x);
     }
 #if __AVX__
-    __m256 func_pack8(const __m256& x) const
+    NCNN_FORCEINLINE __m256 func_pack8(const __m256& x) const
     {
         return tan256_ps(x);
     }
 #if __AVX512F__
-    __m512 func_pack16(const __m512& x) const
+    NCNN_FORCEINLINE __m512 func_pack16(const __m512& x) const
     {
         return tan512_ps(x);
     }
@@ -489,47 +417,24 @@ struct unary_op_tan
 
 struct unary_op_asin
 {
-    float func(const float& x) const
+    NCNN_FORCEINLINE float func(const float& x) const
     {
-        return (float)asin(x);
+        return (float)asinf(x);
     }
 #if __SSE2__
-    __m128 func_pack4(const __m128& x) const
+    NCNN_FORCEINLINE __m128 func_pack4(const __m128& x) const
     {
-        //TODO sse optimize
-        float tmp[4];
-        _mm_storeu_ps(tmp, x);
-        tmp[0] = asin(tmp[0]);
-        tmp[1] = asin(tmp[1]);
-        tmp[2] = asin(tmp[2]);
-        tmp[3] = asin(tmp[3]);
-        return _mm_loadu_ps(tmp);
+        return asin_ps(x);
     }
 #if __AVX__
-    __m256 func_pack8(const __m256& x) const
+    NCNN_FORCEINLINE __m256 func_pack8(const __m256& x) const
     {
-        //TODO avx optimize
-        float tmp[8];
-        _mm256_storeu_ps(tmp, x);
-        tmp[0] = asin(tmp[0]);
-        tmp[1] = asin(tmp[1]);
-        tmp[2] = asin(tmp[2]);
-        tmp[3] = asin(tmp[3]);
-        tmp[4] = asin(tmp[4]);
-        tmp[5] = asin(tmp[5]);
-        tmp[6] = asin(tmp[6]);
-        tmp[7] = asin(tmp[7]);
-        return _mm256_loadu_ps(tmp);
+        return asin256_ps(x);
     }
 #if __AVX512F__
-    __m512 func_pack16(const __m512& x) const
+    NCNN_FORCEINLINE __m512 func_pack16(const __m512& x) const
     {
-        //TODO avx512 optimize
-        float tmp[16];
-        _mm512_storeu_ps(tmp, x);
-        for (int i = 0; i < 16; i++)
-            tmp[i] = asin(tmp[i]);
-        return _mm512_loadu_ps(tmp);
+        return asin512_ps(x);
     }
 #endif // __AVX512F__
 #endif // __AVX__
@@ -538,47 +443,24 @@ struct unary_op_asin
 
 struct unary_op_acos
 {
-    float func(const float& x) const
+    NCNN_FORCEINLINE float func(const float& x) const
     {
-        return (float)acos(x);
+        return (float)acosf(x);
     }
 #if __SSE2__
-    __m128 func_pack4(const __m128& x) const
+    NCNN_FORCEINLINE __m128 func_pack4(const __m128& x) const
     {
-        //TODO sse optimize
-        float tmp[4];
-        _mm_storeu_ps(tmp, x);
-        tmp[0] = acos(tmp[0]);
-        tmp[1] = acos(tmp[1]);
-        tmp[2] = acos(tmp[2]);
-        tmp[3] = acos(tmp[3]);
-        return _mm_loadu_ps(tmp);
+        return acos_ps(x);
     }
 #if __AVX__
-    __m256 func_pack8(const __m256& x) const
+    NCNN_FORCEINLINE __m256 func_pack8(const __m256& x) const
     {
-        //TODO avx optimize
-        float tmp[8];
-        _mm256_storeu_ps(tmp, x);
-        tmp[0] = acos(tmp[0]);
-        tmp[1] = acos(tmp[1]);
-        tmp[2] = acos(tmp[2]);
-        tmp[3] = acos(tmp[3]);
-        tmp[4] = acos(tmp[4]);
-        tmp[5] = acos(tmp[5]);
-        tmp[6] = acos(tmp[6]);
-        tmp[7] = acos(tmp[7]);
-        return _mm256_loadu_ps(tmp);
+        return acos256_ps(x);
     }
 #if __AVX512F__
-    __m512 func_pack16(const __m512& x) const
+    NCNN_FORCEINLINE __m512 func_pack16(const __m512& x) const
     {
-        //TODO avx512 optimize
-        float tmp[16];
-        _mm512_storeu_ps(tmp, x);
-        for (int i = 0; i < 16; i++)
-            tmp[i] = acos(tmp[i]);
-        return _mm512_loadu_ps(tmp);
+        return acos512_ps(x);
     }
 #endif // __AVX512F__
 #endif // __AVX__
@@ -587,47 +469,24 @@ struct unary_op_acos
 
 struct unary_op_atan
 {
-    float func(const float& x) const
+    NCNN_FORCEINLINE float func(const float& x) const
     {
-        return (float)atan(x);
+        return (float)atanf(x);
     }
 #if __SSE2__
-    __m128 func_pack4(const __m128& x) const
+    NCNN_FORCEINLINE __m128 func_pack4(const __m128& x) const
     {
-        //TODO sse optimize
-        float tmp[4];
-        _mm_storeu_ps(tmp, x);
-        tmp[0] = atan(tmp[0]);
-        tmp[1] = atan(tmp[1]);
-        tmp[2] = atan(tmp[2]);
-        tmp[3] = atan(tmp[3]);
-        return _mm_loadu_ps(tmp);
+        return atan_ps(x);
     }
 #if __AVX__
-    __m256 func_pack8(const __m256& x) const
+    NCNN_FORCEINLINE __m256 func_pack8(const __m256& x) const
     {
-        //TODO avx optimize
-        float tmp[8];
-        _mm256_storeu_ps(tmp, x);
-        tmp[0] = atan(tmp[0]);
-        tmp[1] = atan(tmp[1]);
-        tmp[2] = atan(tmp[2]);
-        tmp[3] = atan(tmp[3]);
-        tmp[4] = atan(tmp[4]);
-        tmp[5] = atan(tmp[5]);
-        tmp[6] = atan(tmp[6]);
-        tmp[7] = atan(tmp[7]);
-        return _mm256_loadu_ps(tmp);
+        return atan256_ps(x);
     }
 #if __AVX512F__
-    __m512 func_pack16(const __m512& x) const
+    NCNN_FORCEINLINE __m512 func_pack16(const __m512& x) const
     {
-        //TODO avx512 optimize
-        float tmp[16];
-        _mm512_storeu_ps(tmp, x);
-        for (int i = 0; i < 16; i++)
-            tmp[i] = atan(tmp[i]);
-        return _mm512_loadu_ps(tmp);
+        return atan512_ps(x);
     }
 #endif // __AVX512F__
 #endif // __AVX__
@@ -636,22 +495,22 @@ struct unary_op_atan
 
 struct unary_op_reciprocal
 {
-    float func(const float& x) const
+    NCNN_FORCEINLINE float func(const float& x) const
     {
         return 1.f / x;
     }
 #if __SSE2__
-    __m128 func_pack4(const __m128& x) const
+    NCNN_FORCEINLINE __m128 func_pack4(const __m128& x) const
     {
         return _mm_div_ps(*(__m128*)_ps_1, x);
     }
 #if __AVX__
-    __m256 func_pack8(const __m256& x) const
+    NCNN_FORCEINLINE __m256 func_pack8(const __m256& x) const
     {
         return _mm256_div_ps(*(__m256*)_ps256_1, x);
     }
 #if __AVX512F__
-    __m512 func_pack16(const __m512& x) const
+    NCNN_FORCEINLINE __m512 func_pack16(const __m512& x) const
     {
         return _mm512_div_ps(*(__m512*)_ps512_1, x);
     }
@@ -662,22 +521,22 @@ struct unary_op_reciprocal
 
 struct unary_op_tanh
 {
-    float func(const float& x) const
+    NCNN_FORCEINLINE float func(const float& x) const
     {
-        return (float)tanh(x);
+        return (float)tanhf(x);
     }
 #if __SSE2__
-    __m128 func_pack4(const __m128& x) const
+    NCNN_FORCEINLINE __m128 func_pack4(const __m128& x) const
     {
         return tanh_sse(x);
     }
 #if __AVX__
-    __m256 func_pack8(const __m256& x) const
+    NCNN_FORCEINLINE __m256 func_pack8(const __m256& x) const
     {
         return tanh_avx(x);
     }
 #if __AVX512F__
-    __m512 func_pack16(const __m512& x) const
+    NCNN_FORCEINLINE __m512 func_pack16(const __m512& x) const
     {
         return tanh_avx512(x);
     }
@@ -688,22 +547,22 @@ struct unary_op_tanh
 
 struct unary_op_log10
 {
-    float func(const float& x) const
+    NCNN_FORCEINLINE float func(const float& x) const
     {
-        return (float)log10(x);
+        return (float)log10f(x);
     }
 #if __SSE2__
-    __m128 func_pack4(const __m128& x) const
+    NCNN_FORCEINLINE __m128 func_pack4(const __m128& x) const
     {
         return _mm_mul_ps(log_ps(x), _mm_set1_ps(0.434294481903));
     }
 #if __AVX__
-    __m256 func_pack8(const __m256& x) const
+    NCNN_FORCEINLINE __m256 func_pack8(const __m256& x) const
     {
         return _mm256_mul_ps(log256_ps(x), _mm256_set1_ps(0.434294481903));
     }
 #if __AVX512F__
-    __m512 func_pack16(const __m512& x) const
+    NCNN_FORCEINLINE __m512 func_pack16(const __m512& x) const
     {
         return _mm512_mul_ps(log512_ps(x), _mm512_set1_ps(0.434294481903));
     }
@@ -712,6 +571,76 @@ struct unary_op_log10
 #endif // __SSE2__
 };
 
+struct unary_op_round
+{
+    NCNN_FORCEINLINE float func(const float& x) const
+    {
+        // round to nearest even
+        // return (x + 12582912.f) - 12582912.f;
+#ifdef FE_TONEAREST
+        int old_rm = fegetround();
+        fesetround(FE_TONEAREST);
+#endif
+        float y = nearbyintf(x);
+#ifdef FE_TONEAREST
+        fesetround(old_rm);
+#endif
+        return y;
+    }
+#if __SSE2__
+    NCNN_FORCEINLINE __m128 func_pack4(const __m128& x) const
+    {
+#if __SSE4_1__
+        return _mm_round_ps(x, _MM_FROUND_TO_NEAREST_INT | _MM_FROUND_NO_EXC);
+#else
+        return _mm_cvtepi32_ps(_mm_cvtps_epi32(x));
+#endif
+    }
+#if __AVX__
+    NCNN_FORCEINLINE __m256 func_pack8(const __m256& x) const
+    {
+        return _mm256_round_ps(x, _MM_FROUND_TO_NEAREST_INT | _MM_FROUND_NO_EXC);
+    }
+#if __AVX512F__
+    NCNN_FORCEINLINE __m512 func_pack16(const __m512& x) const
+    {
+        return _mm512_roundscale_ps(x, _MM_FROUND_TO_NEAREST_INT | _MM_FROUND_NO_EXC);
+    }
+#endif // __AVX512F__
+#endif // __AVX__
+#endif // __SSE2__
+};
+
+struct unary_op_trunc
+{
+    NCNN_FORCEINLINE float func(const float& x) const
+    {
+        return (float)truncf(x);
+    }
+#if __SSE2__
+    NCNN_FORCEINLINE __m128 func_pack4(const __m128& x) const
+    {
+#if __SSE4_1__
+        return _mm_round_ps(x, _MM_FROUND_TO_ZERO | _MM_FROUND_NO_EXC);
+#else
+        return _mm_cvtepi32_ps(_mm_cvttps_epi32(x));
+#endif
+    }
+#if __AVX__
+    NCNN_FORCEINLINE __m256 func_pack8(const __m256& x) const
+    {
+        return _mm256_round_ps(x, _MM_FROUND_TO_ZERO | _MM_FROUND_NO_EXC);
+    }
+#if __AVX512F__
+    NCNN_FORCEINLINE __m512 func_pack16(const __m512& x) const
+    {
+        return _mm512_roundscale_ps(x, _MM_FROUND_TO_ZERO | _MM_FROUND_NO_EXC);
+    }
+#endif // __AVX512F__
+#endif // __AVX__
+#endif // __SSE2__
+};
+
 } // namespace UnaryOp_x86_functor
 
 int UnaryOp_x86::forward_inplace(Mat& bottom_top_blob, const Option& opt) const
@@ -771,6 +700,12 @@ int UnaryOp_x86::forward_inplace(Mat& bottom_top_blob, const Option& opt) const
     if (op_type == Operation_LOG10)
         return unary_op_inplace<unary_op_log10>(bottom_top_blob, opt);
 
+    if (op_type == Operation_ROUND)
+        return unary_op_inplace<unary_op_round>(bottom_top_blob, opt);
+
+    if (op_type == Operation_TRUNC)
+        return unary_op_inplace<unary_op_trunc>(bottom_top_blob, opt);
+
     return 0;
 }
 
