@@ -15,15 +15,17 @@
 #ifndef X86_USABILITY_H
 #define X86_USABILITY_H
 
-#include <math.h>
+#include <stdint.h>
 #if __SSE2__
 #include <emmintrin.h>
+#if __SSSE3__
+#include <tmmintrin.h>
 #if __SSE4_1__
 #include <smmintrin.h>
 #if __AVX__
 #include <immintrin.h>
 #if __XOP__
-#ifdef _MSC_VER
+#if defined(_MSC_VER) && !defined(__clang__)
 #include <ammintrin.h>
 #else
 #include <x86intrin.h>
@@ -31,6 +33,7 @@
 #endif
 #endif
 #endif
+#endif
 #endif // __SSE2__
 
 static NCNN_FORCEINLINE signed char float2int8(float v)
@@ -42,6 +45,83 @@ static NCNN_FORCEINLINE signed char float2int8(float v)
 }
 
 #if __SSE2__
+static NCNN_FORCEINLINE void transpose4x8_epi32(__m128i& _r0, __m128i& _r1, __m128i& _r2, __m128i& _r3, __m128i& _r4, __m128i& _r5, __m128i& _r6, __m128i& _r7)
+{
+    __m128i _tmp0 = _mm_unpacklo_epi32(_r0, _r1);
+    __m128i _tmp1 = _mm_unpackhi_epi32(_r0, _r1);
+    __m128i _tmp2 = _mm_unpacklo_epi32(_r2, _r3);
+    __m128i _tmp3 = _mm_unpackhi_epi32(_r2, _r3);
+    __m128i _tmp4 = _mm_unpacklo_epi32(_r4, _r5);
+    __m128i _tmp5 = _mm_unpackhi_epi32(_r4, _r5);
+    __m128i _tmp6 = _mm_unpacklo_epi32(_r6, _r7);
+    __m128i _tmp7 = _mm_unpackhi_epi32(_r6, _r7);
+
+    _r0 = _mm_unpacklo_epi64(_tmp0, _tmp2);
+    _r1 = _mm_unpacklo_epi64(_tmp4, _tmp6);
+    _r2 = _mm_unpackhi_epi64(_tmp0, _tmp2);
+    _r3 = _mm_unpackhi_epi64(_tmp4, _tmp6);
+    _r4 = _mm_unpacklo_epi64(_tmp1, _tmp3);
+    _r5 = _mm_unpacklo_epi64(_tmp5, _tmp7);
+    _r6 = _mm_unpackhi_epi64(_tmp1, _tmp3);
+    _r7 = _mm_unpackhi_epi64(_tmp5, _tmp7);
+}
+
+static NCNN_FORCEINLINE void transpose4x4_epi32(__m128i& _r0, __m128i& _r1, __m128i& _r2, __m128i& _r3)
+{
+    __m128i _tmp0 = _mm_unpacklo_epi32(_r0, _r1);
+    __m128i _tmp1 = _mm_unpackhi_epi32(_r0, _r1);
+    __m128i _tmp2 = _mm_unpacklo_epi32(_r2, _r3);
+    __m128i _tmp3 = _mm_unpackhi_epi32(_r2, _r3);
+
+    _r0 = _mm_unpacklo_epi64(_tmp0, _tmp2);
+    _r1 = _mm_unpackhi_epi64(_tmp0, _tmp2);
+    _r2 = _mm_unpacklo_epi64(_tmp1, _tmp3);
+    _r3 = _mm_unpackhi_epi64(_tmp1, _tmp3);
+}
+
+static NCNN_FORCEINLINE void transpose8x8_epi16(__m128i& _r0, __m128i& _r1, __m128i& _r2, __m128i& _r3, __m128i& _r4, __m128i& _r5, __m128i& _r6, __m128i& _r7)
+{
+    __m128i _tmp0 = _mm_unpacklo_epi16(_r0, _r1);
+    __m128i _tmp1 = _mm_unpackhi_epi16(_r0, _r1);
+    __m128i _tmp2 = _mm_unpacklo_epi16(_r2, _r3);
+    __m128i _tmp3 = _mm_unpackhi_epi16(_r2, _r3);
+    __m128i _tmp4 = _mm_unpacklo_epi16(_r4, _r5);
+    __m128i _tmp5 = _mm_unpackhi_epi16(_r4, _r5);
+    __m128i _tmp6 = _mm_unpacklo_epi16(_r6, _r7);
+    __m128i _tmp7 = _mm_unpackhi_epi16(_r6, _r7);
+
+    __m128i _tmp8 = _mm_unpacklo_epi32(_tmp0, _tmp2);
+    __m128i _tmp9 = _mm_unpackhi_epi32(_tmp0, _tmp2);
+    __m128i _tmpa = _mm_unpacklo_epi32(_tmp1, _tmp3);
+    __m128i _tmpb = _mm_unpackhi_epi32(_tmp1, _tmp3);
+    __m128i _tmpc = _mm_unpacklo_epi32(_tmp4, _tmp6);
+    __m128i _tmpd = _mm_unpackhi_epi32(_tmp4, _tmp6);
+    __m128i _tmpe = _mm_unpacklo_epi32(_tmp5, _tmp7);
+    __m128i _tmpf = _mm_unpackhi_epi32(_tmp5, _tmp7);
+
+    _r0 = _mm_unpacklo_epi64(_tmp8, _tmpc);
+    _r1 = _mm_unpackhi_epi64(_tmp8, _tmpc);
+    _r2 = _mm_unpacklo_epi64(_tmp9, _tmpd);
+    _r3 = _mm_unpackhi_epi64(_tmp9, _tmpd);
+    _r4 = _mm_unpacklo_epi64(_tmpa, _tmpe);
+    _r5 = _mm_unpackhi_epi64(_tmpa, _tmpe);
+    _r6 = _mm_unpacklo_epi64(_tmpb, _tmpf);
+    _r7 = _mm_unpackhi_epi64(_tmpb, _tmpf);
+}
+
+static NCNN_FORCEINLINE void transpose8x4_epi16(__m128i& _r0, __m128i& _r1, __m128i& _r2, __m128i& _r3)
+{
+    __m128i _tmp0 = _mm_unpacklo_epi16(_r0, _r1);
+    __m128i _tmp1 = _mm_unpackhi_epi16(_r0, _r1);
+    __m128i _tmp2 = _mm_unpacklo_epi16(_r2, _r3);
+    __m128i _tmp3 = _mm_unpackhi_epi16(_r2, _r3);
+
+    _r0 = _mm_unpacklo_epi32(_tmp0, _tmp2);
+    _r1 = _mm_unpackhi_epi32(_tmp0, _tmp2);
+    _r2 = _mm_unpacklo_epi32(_tmp1, _tmp3);
+    _r3 = _mm_unpackhi_epi32(_tmp1, _tmp3);
+}
+
 static NCNN_FORCEINLINE float _mm_reduce_add_ps(__m128 x128)
 {
     const __m128 x64 = _mm_add_ps(x128, _mm_movehl_ps(x128, x128));
@@ -196,6 +276,14 @@ static NCNN_FORCEINLINE __m128 _mm_comp_fnmadd_ps(const __m128& _a, const __m128
 {
     return _mm_sub_ps(_c, _mm_mul_ps(_a, _b));
 }
+static NCNN_FORCEINLINE __m128 _mm_comp_fmsub_ps(const __m128& _a, const __m128& _b, const __m128& _c)
+{
+    return _mm_sub_ps(_mm_mul_ps(_a, _b), _c);
+}
+static NCNN_FORCEINLINE __m128 _mm_comp_fnmsub_ps(const __m128& _a, const __m128& _b, const __m128& _c)
+{
+    return _mm_sub_ps(_c, _mm_mul_ps(_mm_mul_ps(_a, _b), _mm_set1_ps(-1)));
+}
 #else
 static NCNN_FORCEINLINE __m128 _mm_comp_fmadd_ps(const __m128& _a, const __m128& _b, const __m128& _c)
 {
@@ -206,6 +294,14 @@ static NCNN_FORCEINLINE __m128 _mm_comp_fnmadd_ps(const __m128& _a, const __m128
     // return -a * b + c
     return _mm_fnmadd_ps(_a, _b, _c);
 }
+static NCNN_FORCEINLINE __m128 _mm_comp_fmsub_ps(const __m128& _a, const __m128& _b, const __m128& _c)
+{
+    return _mm_fmsub_ps(_a, _b, _c);
+}
+static NCNN_FORCEINLINE __m128 _mm_comp_fnmsub_ps(const __m128& _a, const __m128& _b, const __m128& _c)
+{
+    return _mm_fnmsub_ps(_a, _b, _c);
+}
 #endif // !__FMA__
 
 #if __AVX__
@@ -218,9 +314,18 @@ static NCNN_FORCEINLINE __m256 _mm256_comp_fnmadd_ps(const __m256& _a, const __m
 {
     return _mm256_sub_ps(_c, _mm256_mul_ps(_a, _b));
 }
+static NCNN_FORCEINLINE __m256 _mm256_comp_fmsub_ps(const __m256& _a, const __m256& _b, const __m256& _c)
+{
+    return _mm256_sub_ps(_mm256_mul_ps(_a, _b), _c);
+}
+static NCNN_FORCEINLINE __m256 _mm256_comp_fnmsub_ps(const __m256& _a, const __m256& _b, const __m256& _c)
+{
+    return _mm256_sub_ps(_c, _mm256_mul_ps(_mm256_mul_ps(_a, _b), _mm256_set1_ps(-1)));
+}
 #else
 static NCNN_FORCEINLINE __m256 _mm256_comp_fmadd_ps(const __m256& _a, const __m256& _b, const __m256& _c)
 {
+    // return a * b + c
     return _mm256_fmadd_ps(_a, _b, _c);
 }
 static NCNN_FORCEINLINE __m256 _mm256_comp_fnmadd_ps(const __m256& _a, const __m256& _b, const __m256& _c)
@@ -228,6 +333,16 @@ static NCNN_FORCEINLINE __m256 _mm256_comp_fnmadd_ps(const __m256& _a, const __m
     // return -a * b + c
     return _mm256_fnmadd_ps(_a, _b, _c);
 }
+static NCNN_FORCEINLINE __m256 _mm256_comp_fmsub_ps(const __m256& _a, const __m256& _b, const __m256& _c)
+{
+    // return a * b - c
+    return _mm256_fmsub_ps(_a, _b, _c);
+}
+static NCNN_FORCEINLINE __m256 _mm256_comp_fnmsub_ps(const __m256& _a, const __m256& _b, const __m256& _c)
+{
+    // return -(a * b) - c
+    return _mm256_fnmsub_ps(_a, _b, _c);
+}
 #endif
 
 static NCNN_FORCEINLINE __m256 _mm256_fmadd_1_ps(const __m256& a, const __m256& b, float c)
@@ -341,34 +456,161 @@ static NCNN_FORCEINLINE void transpose8x2_ps(__m256& _r0, __m256& _r1)
     _r1 = _mm256_permute2f128_ps(_tmp0, _tmp1, _MM_SHUFFLE(0, 3, 0, 1));
 }
 
-static NCNN_FORCEINLINE void transpose8x8_epi16(__m128i& _r0, __m128i& _r1, __m128i& _r2, __m128i& _r3, __m128i& _r4, __m128i& _r5, __m128i& _r6, __m128i& _r7)
+static NCNN_FORCEINLINE void transpose2x8_ps(__m256& _r0, __m256& _r1)
 {
-    __m128i _tmp0 = _mm_unpacklo_epi16(_r0, _r1);
-    __m128i _tmp1 = _mm_unpackhi_epi16(_r0, _r1);
-    __m128i _tmp2 = _mm_unpacklo_epi16(_r2, _r3);
-    __m128i _tmp3 = _mm_unpackhi_epi16(_r2, _r3);
-    __m128i _tmp4 = _mm_unpacklo_epi16(_r4, _r5);
-    __m128i _tmp5 = _mm_unpackhi_epi16(_r4, _r5);
-    __m128i _tmp6 = _mm_unpacklo_epi16(_r6, _r7);
-    __m128i _tmp7 = _mm_unpackhi_epi16(_r6, _r7);
+    __m256 _tmp0 = _mm256_permute2f128_ps(_r0, _r1, _MM_SHUFFLE(0, 2, 0, 0));
+    __m256 _tmp1 = _mm256_permute2f128_ps(_r0, _r1, _MM_SHUFFLE(0, 3, 0, 1));
 
-    __m128i _tmp8 = _mm_unpacklo_epi32(_tmp0, _tmp2);
-    __m128i _tmp9 = _mm_unpackhi_epi32(_tmp0, _tmp2);
-    __m128i _tmpa = _mm_unpacklo_epi32(_tmp1, _tmp3);
-    __m128i _tmpb = _mm_unpackhi_epi32(_tmp1, _tmp3);
-    __m128i _tmpc = _mm_unpacklo_epi32(_tmp4, _tmp6);
-    __m128i _tmpd = _mm_unpackhi_epi32(_tmp4, _tmp6);
-    __m128i _tmpe = _mm_unpacklo_epi32(_tmp5, _tmp7);
-    __m128i _tmpf = _mm_unpackhi_epi32(_tmp5, _tmp7);
+    _r0 = _mm256_shuffle_ps(_tmp0, _tmp1, _MM_SHUFFLE(2, 0, 2, 0));
+    _r1 = _mm256_shuffle_ps(_tmp0, _tmp1, _MM_SHUFFLE(3, 1, 3, 1));
+}
 
-    _r0 = _mm_unpacklo_epi64(_tmp8, _tmpc);
-    _r1 = _mm_unpackhi_epi64(_tmp8, _tmpc);
-    _r2 = _mm_unpacklo_epi64(_tmp9, _tmpd);
-    _r3 = _mm_unpackhi_epi64(_tmp9, _tmpd);
-    _r4 = _mm_unpacklo_epi64(_tmpa, _tmpe);
-    _r5 = _mm_unpackhi_epi64(_tmpa, _tmpe);
-    _r6 = _mm_unpacklo_epi64(_tmpb, _tmpf);
-    _r7 = _mm_unpackhi_epi64(_tmpb, _tmpf);
+static NCNN_FORCEINLINE void transpose3x8_ps(__m256& _r0, __m256& _r1, __m256& _r2)
+{
+    __m256 _tmp0 = _mm256_permute2f128_ps(_r0, _r1, _MM_SHUFFLE(0, 3, 0, 0));
+    __m256 _tmp1 = _mm256_permute2f128_ps(_r0, _r2, _MM_SHUFFLE(0, 2, 0, 1));
+    __m256 _tmp2 = _mm256_permute2f128_ps(_r1, _r2, _MM_SHUFFLE(0, 3, 0, 0));
+
+    __m256 _tmp4 = _mm256_shuffle_ps(_tmp0, _tmp1, _MM_SHUFFLE(1, 0, 2, 1));
+    __m256 _tmp5 = _mm256_shuffle_ps(_tmp1, _tmp2, _MM_SHUFFLE(2, 1, 3, 2));
+
+    _r0 = _mm256_shuffle_ps(_tmp0, _tmp5, _MM_SHUFFLE(2, 0, 3, 0));
+    _r1 = _mm256_shuffle_ps(_tmp4, _tmp5, _MM_SHUFFLE(3, 1, 2, 0));
+    _r2 = _mm256_shuffle_ps(_tmp4, _tmp2, _MM_SHUFFLE(3, 0, 3, 1));
+}
+
+static NCNN_FORCEINLINE void transpose8x6_ps(__m256& _r0, __m256& _r1, __m256& _r2, __m256& _r3, __m256& _r4, __m256& _r5)
+{
+    __m256 _tmp0 = _mm256_unpacklo_ps(_r0, _r1);
+    __m256 _tmp1 = _mm256_unpackhi_ps(_r0, _r1);
+    __m256 _tmp2 = _mm256_unpacklo_ps(_r2, _r3);
+    __m256 _tmp3 = _mm256_unpackhi_ps(_r2, _r3);
+    __m256 _tmp4 = _mm256_unpacklo_ps(_r4, _r5);
+    __m256 _tmp5 = _mm256_unpackhi_ps(_r4, _r5);
+
+    __m256 _tmp6 = _mm256_shuffle_ps(_tmp0, _tmp2, _MM_SHUFFLE(1, 0, 1, 0));
+    __m256 _tmp7 = _mm256_shuffle_ps(_tmp4, _tmp0, _MM_SHUFFLE(3, 2, 1, 0));
+    __m256 _tmp8 = _mm256_shuffle_ps(_tmp2, _tmp4, _MM_SHUFFLE(3, 2, 3, 2));
+    __m256 _tmp9 = _mm256_shuffle_ps(_tmp1, _tmp3, _MM_SHUFFLE(1, 0, 1, 0));
+    __m256 _tmpa = _mm256_shuffle_ps(_tmp5, _tmp1, _MM_SHUFFLE(3, 2, 1, 0));
+    __m256 _tmpb = _mm256_shuffle_ps(_tmp3, _tmp5, _MM_SHUFFLE(3, 2, 3, 2));
+
+    _r0 = _mm256_permute2f128_ps(_tmp6, _tmp7, _MM_SHUFFLE(0, 2, 0, 0));
+    _r1 = _mm256_permute2f128_ps(_tmp8, _tmp9, _MM_SHUFFLE(0, 2, 0, 0));
+    _r2 = _mm256_permute2f128_ps(_tmpa, _tmpb, _MM_SHUFFLE(0, 2, 0, 0));
+    _r3 = _mm256_permute2f128_ps(_tmp6, _tmp7, _MM_SHUFFLE(0, 3, 0, 1));
+    _r4 = _mm256_permute2f128_ps(_tmp8, _tmp9, _MM_SHUFFLE(0, 3, 0, 1));
+    _r5 = _mm256_permute2f128_ps(_tmpa, _tmpb, _MM_SHUFFLE(0, 3, 0, 1));
+}
+
+static NCNN_FORCEINLINE void transpose8x11_ps(__m256& _r0, __m256& _r1, __m256& _r2, __m256& _r3, __m256& _r4, __m256& _r5, __m256& _r6, __m256& _r7, __m256& _r8, __m256& _r9, __m256& _ra)
+{
+    __m256 _tmp0 = _mm256_unpacklo_ps(_r0, _r1);
+    __m256 _tmp1 = _mm256_unpackhi_ps(_r0, _r1);
+    __m256 _tmp2 = _mm256_unpacklo_ps(_r2, _r3);
+    __m256 _tmp3 = _mm256_unpackhi_ps(_r2, _r3);
+    __m256 _tmp4 = _mm256_unpacklo_ps(_r4, _r5);
+    __m256 _tmp5 = _mm256_unpackhi_ps(_r4, _r5);
+    __m256 _tmp6 = _mm256_unpacklo_ps(_r6, _r7);
+    __m256 _tmp7 = _mm256_unpackhi_ps(_r6, _r7);
+    __m256 _tmp8 = _mm256_unpacklo_ps(_r8, _r9);
+    __m256 _tmp9 = _mm256_unpackhi_ps(_r8, _r9);
+    __m256 _tmpa = _mm256_unpacklo_ps(_ra, _r0);
+    __m256 _tmpb = _mm256_shuffle_ps(_ra, _tmp1, _MM_SHUFFLE(3, 2, 1, 2));
+    __m256 _tmpc = _mm256_unpacklo_ps(_r1, _r2);
+    __m256 _tmpd = _mm256_unpackhi_ps(_r1, _r2);
+    __m256 _tmpe = _mm256_unpacklo_ps(_r3, _r4);
+    __m256 _tmpf = _mm256_unpackhi_ps(_r3, _r4);
+    __m256 _tmpg = _mm256_unpacklo_ps(_r5, _r6);
+    __m256 _tmph = _mm256_unpackhi_ps(_r5, _r6);
+    __m256 _tmpi = _mm256_unpacklo_ps(_r7, _r8);
+    __m256 _tmpj = _mm256_unpackhi_ps(_r7, _r8);
+    __m256 _tmpk = _mm256_unpacklo_ps(_r9, _ra);
+    __m256 _tmpl = _mm256_unpackhi_ps(_r9, _ra);
+
+    __m256 _tmpm = _mm256_shuffle_ps(_tmp0, _tmp2, _MM_SHUFFLE(1, 0, 1, 0));
+    __m256 _tmpn = _mm256_shuffle_ps(_tmp4, _tmp6, _MM_SHUFFLE(1, 0, 1, 0));
+    __m256 _tmpo = _mm256_shuffle_ps(_tmp8, _tmpa, _MM_SHUFFLE(3, 0, 1, 0));
+    __m256 _tmpp = _mm256_shuffle_ps(_tmpc, _tmpe, _MM_SHUFFLE(3, 2, 3, 2));
+    __m256 _tmpq = _mm256_shuffle_ps(_tmpg, _tmpi, _MM_SHUFFLE(3, 2, 3, 2));
+    __m256 _tmpr = _mm256_shuffle_ps(_tmpk, _tmp1, _MM_SHUFFLE(1, 0, 3, 2));
+    __m256 _tmps = _mm256_shuffle_ps(_tmp3, _tmp5, _MM_SHUFFLE(1, 0, 1, 0));
+    __m256 _tmpt = _mm256_shuffle_ps(_tmp7, _tmp9, _MM_SHUFFLE(1, 0, 1, 0));
+    __m256 _tmpu = _mm256_shuffle_ps(_tmpb, _tmpd, _MM_SHUFFLE(3, 2, 2, 0));
+    __m256 _tmpv = _mm256_shuffle_ps(_tmpf, _tmph, _MM_SHUFFLE(3, 2, 3, 2));
+    __m256 _tmpw = _mm256_shuffle_ps(_tmpj, _tmpl, _MM_SHUFFLE(3, 2, 3, 2));
+
+    _r0 = _mm256_permute2f128_ps(_tmpm, _tmpn, _MM_SHUFFLE(0, 2, 0, 0));
+    _r1 = _mm256_permute2f128_ps(_tmpo, _tmpp, _MM_SHUFFLE(0, 2, 0, 0));
+    _r2 = _mm256_permute2f128_ps(_tmpq, _tmpr, _MM_SHUFFLE(0, 2, 0, 0));
+    _r3 = _mm256_permute2f128_ps(_tmps, _tmpt, _MM_SHUFFLE(0, 2, 0, 0));
+    _r4 = _mm256_permute2f128_ps(_tmpu, _tmpv, _MM_SHUFFLE(0, 2, 0, 0));
+    _r5 = _mm256_permute2f128_ps(_tmpw, _tmpm, _MM_SHUFFLE(0, 3, 0, 0));
+    _r6 = _mm256_permute2f128_ps(_tmpn, _tmpo, _MM_SHUFFLE(0, 3, 0, 1));
+    _r7 = _mm256_permute2f128_ps(_tmpp, _tmpq, _MM_SHUFFLE(0, 3, 0, 1));
+    _r8 = _mm256_permute2f128_ps(_tmpr, _tmps, _MM_SHUFFLE(0, 3, 0, 1));
+    _r9 = _mm256_permute2f128_ps(_tmpt, _tmpu, _MM_SHUFFLE(0, 3, 0, 1));
+    _ra = _mm256_permute2f128_ps(_tmpv, _tmpw, _MM_SHUFFLE(0, 3, 0, 1));
+}
+
+static void transpose8x18_ps(__m256& _r0, __m256& _r1, __m256& _r2, __m256& _r3, __m256& _r4, __m256& _r5, __m256& _r6, __m256& _r7, __m256& _r8, __m256& _r9, __m256& _ra, __m256& _rb, __m256& _rc, __m256& _rd, __m256& _re, __m256& _rf, __m256& _rg, __m256& _rh)
+{
+    __m256 _tmp0 = _mm256_unpacklo_ps(_r0, _r1);
+    __m256 _tmp1 = _mm256_unpackhi_ps(_r0, _r1);
+    __m256 _tmp2 = _mm256_unpacklo_ps(_r2, _r3);
+    __m256 _tmp3 = _mm256_unpackhi_ps(_r2, _r3);
+    __m256 _tmp4 = _mm256_unpacklo_ps(_r4, _r5);
+    __m256 _tmp5 = _mm256_unpackhi_ps(_r4, _r5);
+    __m256 _tmp6 = _mm256_unpacklo_ps(_r6, _r7);
+    __m256 _tmp7 = _mm256_unpackhi_ps(_r6, _r7);
+    __m256 _tmp8 = _mm256_unpacklo_ps(_r8, _r9);
+    __m256 _tmp9 = _mm256_unpackhi_ps(_r8, _r9);
+    __m256 _tmpa = _mm256_unpacklo_ps(_ra, _rb);
+    __m256 _tmpb = _mm256_unpackhi_ps(_ra, _rb);
+    __m256 _tmpc = _mm256_unpacklo_ps(_rc, _rd);
+    __m256 _tmpd = _mm256_unpackhi_ps(_rc, _rd);
+    __m256 _tmpe = _mm256_unpacklo_ps(_re, _rf);
+    __m256 _tmpf = _mm256_unpackhi_ps(_re, _rf);
+    __m256 _tmpg = _mm256_unpacklo_ps(_rg, _rh);
+    __m256 _tmph = _mm256_unpackhi_ps(_rg, _rh);
+
+    __m256 _tmpi = _mm256_shuffle_ps(_tmp0, _tmp2, _MM_SHUFFLE(1, 0, 1, 0));
+    __m256 _tmpj = _mm256_shuffle_ps(_tmp4, _tmp6, _MM_SHUFFLE(1, 0, 1, 0));
+    __m256 _tmpk = _mm256_shuffle_ps(_tmp8, _tmpa, _MM_SHUFFLE(1, 0, 1, 0));
+    __m256 _tmpl = _mm256_shuffle_ps(_tmpc, _tmpe, _MM_SHUFFLE(1, 0, 1, 0));
+    __m256 _tmpm = _mm256_shuffle_ps(_tmpg, _tmp0, _MM_SHUFFLE(3, 2, 1, 0));
+    __m256 _tmpn = _mm256_shuffle_ps(_tmp2, _tmp4, _MM_SHUFFLE(3, 2, 3, 2));
+    __m256 _tmpo = _mm256_shuffle_ps(_tmp6, _tmp8, _MM_SHUFFLE(3, 2, 3, 2));
+    __m256 _tmpp = _mm256_shuffle_ps(_tmpa, _tmpc, _MM_SHUFFLE(3, 2, 3, 2));
+    __m256 _tmpq = _mm256_shuffle_ps(_tmpe, _tmpg, _MM_SHUFFLE(3, 2, 3, 2));
+    __m256 _tmpr = _mm256_shuffle_ps(_tmp1, _tmp3, _MM_SHUFFLE(1, 0, 1, 0));
+    __m256 _tmps = _mm256_shuffle_ps(_tmp5, _tmp7, _MM_SHUFFLE(1, 0, 1, 0));
+    __m256 _tmpt = _mm256_shuffle_ps(_tmp9, _tmpb, _MM_SHUFFLE(1, 0, 1, 0));
+    __m256 _tmpu = _mm256_shuffle_ps(_tmpd, _tmpf, _MM_SHUFFLE(1, 0, 1, 0));
+    __m256 _tmpv = _mm256_shuffle_ps(_tmph, _tmp1, _MM_SHUFFLE(3, 2, 1, 0));
+    __m256 _tmpw = _mm256_shuffle_ps(_tmp3, _tmp5, _MM_SHUFFLE(3, 2, 3, 2));
+    __m256 _tmpx = _mm256_shuffle_ps(_tmp7, _tmp9, _MM_SHUFFLE(3, 2, 3, 2));
+    __m256 _tmpy = _mm256_shuffle_ps(_tmpb, _tmpd, _MM_SHUFFLE(3, 2, 3, 2));
+    __m256 _tmpz = _mm256_shuffle_ps(_tmpf, _tmph, _MM_SHUFFLE(3, 2, 3, 2));
+
+    _r0 = _mm256_permute2f128_ps(_tmpi, _tmpj, _MM_SHUFFLE(0, 2, 0, 0));
+    _r1 = _mm256_permute2f128_ps(_tmpk, _tmpl, _MM_SHUFFLE(0, 2, 0, 0));
+    _r2 = _mm256_permute2f128_ps(_tmpm, _tmpn, _MM_SHUFFLE(0, 2, 0, 0));
+    _r3 = _mm256_permute2f128_ps(_tmpo, _tmpp, _MM_SHUFFLE(0, 2, 0, 0));
+    _r4 = _mm256_permute2f128_ps(_tmpq, _tmpr, _MM_SHUFFLE(0, 2, 0, 0));
+    _r5 = _mm256_permute2f128_ps(_tmps, _tmpt, _MM_SHUFFLE(0, 2, 0, 0));
+    _r6 = _mm256_permute2f128_ps(_tmpu, _tmpv, _MM_SHUFFLE(0, 2, 0, 0));
+    _r7 = _mm256_permute2f128_ps(_tmpw, _tmpx, _MM_SHUFFLE(0, 2, 0, 0));
+    _r8 = _mm256_permute2f128_ps(_tmpy, _tmpz, _MM_SHUFFLE(0, 2, 0, 0));
+    _r9 = _mm256_permute2f128_ps(_tmpi, _tmpj, _MM_SHUFFLE(0, 3, 0, 1));
+    _ra = _mm256_permute2f128_ps(_tmpk, _tmpl, _MM_SHUFFLE(0, 3, 0, 1));
+    _rb = _mm256_permute2f128_ps(_tmpm, _tmpn, _MM_SHUFFLE(0, 3, 0, 1));
+    _rc = _mm256_permute2f128_ps(_tmpo, _tmpp, _MM_SHUFFLE(0, 3, 0, 1));
+    _rd = _mm256_permute2f128_ps(_tmpq, _tmpr, _MM_SHUFFLE(0, 3, 0, 1));
+    _re = _mm256_permute2f128_ps(_tmps, _tmpt, _MM_SHUFFLE(0, 3, 0, 1));
+    _rf = _mm256_permute2f128_ps(_tmpu, _tmpv, _MM_SHUFFLE(0, 3, 0, 1));
+    _rg = _mm256_permute2f128_ps(_tmpw, _tmpx, _MM_SHUFFLE(0, 3, 0, 1));
+    _rh = _mm256_permute2f128_ps(_tmpy, _tmpz, _MM_SHUFFLE(0, 3, 0, 1));
 }
 
 static NCNN_FORCEINLINE __m256 HorizontalSums(__m256& v0, __m256& v1, __m256& v2, __m256& v3, __m256& v4, __m256& v5, __m256& v6, __m256& v7)
@@ -598,6 +840,55 @@ static NCNN_FORCEINLINE __m256i float2bfloat_avx(const __m256& v0, const __m256&
     return _v;
 }
 
+#if __AVX2__
+static NCNN_FORCEINLINE void transpose8x2_epi32(__m256i& _r0, __m256i& _r1)
+{
+    __m256i _tmp0 = _mm256_unpacklo_epi32(_r0, _r1);
+    __m256i _tmp1 = _mm256_unpackhi_epi32(_r0, _r1);
+
+    _r0 = _mm256_permute2x128_si256(_tmp0, _tmp1, _MM_SHUFFLE(0, 2, 0, 0));
+    _r1 = _mm256_permute2x128_si256(_tmp0, _tmp1, _MM_SHUFFLE(0, 3, 0, 1));
+}
+
+static NCNN_FORCEINLINE void transpose16x8_epi16(__m256i& _r0, __m256i& _r1, __m256i& _r2, __m256i& _r3, __m256i& _r4, __m256i& _r5, __m256i& _r6, __m256i& _r7)
+{
+    __m256i _tmp0 = _mm256_unpacklo_epi16(_r0, _r1);
+    __m256i _tmp1 = _mm256_unpackhi_epi16(_r0, _r1);
+    __m256i _tmp2 = _mm256_unpacklo_epi16(_r2, _r3);
+    __m256i _tmp3 = _mm256_unpackhi_epi16(_r2, _r3);
+    __m256i _tmp4 = _mm256_unpacklo_epi16(_r4, _r5);
+    __m256i _tmp5 = _mm256_unpackhi_epi16(_r4, _r5);
+    __m256i _tmp6 = _mm256_unpacklo_epi16(_r6, _r7);
+    __m256i _tmp7 = _mm256_unpackhi_epi16(_r6, _r7);
+
+    __m256i _tmpg = _mm256_unpacklo_epi32(_tmp0, _tmp2);
+    __m256i _tmph = _mm256_unpackhi_epi32(_tmp0, _tmp2);
+    __m256i _tmpi = _mm256_unpacklo_epi32(_tmp1, _tmp3);
+    __m256i _tmpj = _mm256_unpackhi_epi32(_tmp1, _tmp3);
+    __m256i _tmpk = _mm256_unpacklo_epi32(_tmp4, _tmp6);
+    __m256i _tmpl = _mm256_unpackhi_epi32(_tmp4, _tmp6);
+    __m256i _tmpm = _mm256_unpacklo_epi32(_tmp5, _tmp7);
+    __m256i _tmpn = _mm256_unpackhi_epi32(_tmp5, _tmp7);
+
+    _tmp0 = _mm256_unpacklo_epi64(_tmpg, _tmpk);
+    _tmp1 = _mm256_unpackhi_epi64(_tmpg, _tmpk);
+    _tmp2 = _mm256_unpacklo_epi64(_tmph, _tmpl);
+    _tmp3 = _mm256_unpackhi_epi64(_tmph, _tmpl);
+    _tmp4 = _mm256_unpacklo_epi64(_tmpi, _tmpm);
+    _tmp5 = _mm256_unpackhi_epi64(_tmpi, _tmpm);
+    _tmp6 = _mm256_unpacklo_epi64(_tmpj, _tmpn);
+    _tmp7 = _mm256_unpackhi_epi64(_tmpj, _tmpn);
+
+    _r0 = _mm256_permute2x128_si256(_tmp0, _tmp1, _MM_SHUFFLE(0, 2, 0, 0));
+    _r1 = _mm256_permute2x128_si256(_tmp2, _tmp3, _MM_SHUFFLE(0, 2, 0, 0));
+    _r2 = _mm256_permute2x128_si256(_tmp4, _tmp5, _MM_SHUFFLE(0, 2, 0, 0));
+    _r3 = _mm256_permute2x128_si256(_tmp6, _tmp7, _MM_SHUFFLE(0, 2, 0, 0));
+    _r4 = _mm256_permute2x128_si256(_tmp0, _tmp1, _MM_SHUFFLE(0, 3, 0, 1));
+    _r5 = _mm256_permute2x128_si256(_tmp2, _tmp3, _MM_SHUFFLE(0, 3, 0, 1));
+    _r6 = _mm256_permute2x128_si256(_tmp4, _tmp5, _MM_SHUFFLE(0, 3, 0, 1));
+    _r7 = _mm256_permute2x128_si256(_tmp6, _tmp7, _MM_SHUFFLE(0, 3, 0, 1));
+}
+
 #if __AVX512F__
 static NCNN_FORCEINLINE void transpose16x16_ps(__m512& _r0, __m512& _r1, __m512& _r2, __m512& _r3, __m512& _r4, __m512& _r5, __m512& _r6, __m512& _r7,
         __m512& _r8, __m512& _r9, __m512& _ra, __m512& _rb, __m512& _rc, __m512& _rd, __m512& _re, __m512& _rf)
@@ -928,43 +1219,58 @@ static NCNN_FORCEINLINE void transpose16x16_epi16(__m256i& _r0, __m256i& _r1, __
     _rf = _mm256_permute2x128_si256(_tmp7, _tmpf, _MM_SHUFFLE(0, 3, 0, 1));
 }
 
-static NCNN_FORCEINLINE void transpose16x8_epi16(__m256i& _r0, __m256i& _r1, __m256i& _r2, __m256i& _r3, __m256i& _r4, __m256i& _r5, __m256i& _r6, __m256i& _r7)
+static NCNN_FORCEINLINE void transpose8x16_epi16(__m128i& _r0, __m128i& _r1, __m128i& _r2, __m128i& _r3, __m128i& _r4, __m128i& _r5, __m128i& _r6, __m128i& _r7, __m128i& _r8, __m128i& _r9, __m128i& _ra, __m128i& _rb, __m128i& _rc, __m128i& _rd, __m128i& _re, __m128i& _rf)
 {
-    __m256i _tmp0 = _mm256_unpacklo_epi16(_r0, _r1);
-    __m256i _tmp1 = _mm256_unpackhi_epi16(_r0, _r1);
-    __m256i _tmp2 = _mm256_unpacklo_epi16(_r2, _r3);
-    __m256i _tmp3 = _mm256_unpackhi_epi16(_r2, _r3);
-    __m256i _tmp4 = _mm256_unpacklo_epi16(_r4, _r5);
-    __m256i _tmp5 = _mm256_unpackhi_epi16(_r4, _r5);
-    __m256i _tmp6 = _mm256_unpacklo_epi16(_r6, _r7);
-    __m256i _tmp7 = _mm256_unpackhi_epi16(_r6, _r7);
+    __m128i _tmp0 = _mm_unpacklo_epi16(_r0, _r1);
+    __m128i _tmp1 = _mm_unpackhi_epi16(_r0, _r1);
+    __m128i _tmp2 = _mm_unpacklo_epi16(_r2, _r3);
+    __m128i _tmp3 = _mm_unpackhi_epi16(_r2, _r3);
+    __m128i _tmp4 = _mm_unpacklo_epi16(_r4, _r5);
+    __m128i _tmp5 = _mm_unpackhi_epi16(_r4, _r5);
+    __m128i _tmp6 = _mm_unpacklo_epi16(_r6, _r7);
+    __m128i _tmp7 = _mm_unpackhi_epi16(_r6, _r7);
+    __m128i _tmp8 = _mm_unpacklo_epi16(_r8, _r9);
+    __m128i _tmp9 = _mm_unpackhi_epi16(_r8, _r9);
+    __m128i _tmpa = _mm_unpacklo_epi16(_ra, _rb);
+    __m128i _tmpb = _mm_unpackhi_epi16(_ra, _rb);
+    __m128i _tmpc = _mm_unpacklo_epi16(_rc, _rd);
+    __m128i _tmpd = _mm_unpackhi_epi16(_rc, _rd);
+    __m128i _tmpe = _mm_unpacklo_epi16(_re, _rf);
+    __m128i _tmpf = _mm_unpackhi_epi16(_re, _rf);
 
-    __m256i _tmpg = _mm256_unpacklo_epi32(_tmp0, _tmp2);
-    __m256i _tmph = _mm256_unpackhi_epi32(_tmp0, _tmp2);
-    __m256i _tmpi = _mm256_unpacklo_epi32(_tmp1, _tmp3);
-    __m256i _tmpj = _mm256_unpackhi_epi32(_tmp1, _tmp3);
-    __m256i _tmpk = _mm256_unpacklo_epi32(_tmp4, _tmp6);
-    __m256i _tmpl = _mm256_unpackhi_epi32(_tmp4, _tmp6);
-    __m256i _tmpm = _mm256_unpacklo_epi32(_tmp5, _tmp7);
-    __m256i _tmpn = _mm256_unpackhi_epi32(_tmp5, _tmp7);
+    __m128i _tmpg = _mm_unpacklo_epi32(_tmp0, _tmp2);
+    __m128i _tmph = _mm_unpackhi_epi32(_tmp0, _tmp2);
+    __m128i _tmpi = _mm_unpacklo_epi32(_tmp1, _tmp3);
+    __m128i _tmpj = _mm_unpackhi_epi32(_tmp1, _tmp3);
+    __m128i _tmpk = _mm_unpacklo_epi32(_tmp4, _tmp6);
+    __m128i _tmpl = _mm_unpackhi_epi32(_tmp4, _tmp6);
+    __m128i _tmpm = _mm_unpacklo_epi32(_tmp5, _tmp7);
+    __m128i _tmpn = _mm_unpackhi_epi32(_tmp5, _tmp7);
+    __m128i _tmpo = _mm_unpacklo_epi32(_tmp8, _tmpa);
+    __m128i _tmpp = _mm_unpackhi_epi32(_tmp8, _tmpa);
+    __m128i _tmpq = _mm_unpacklo_epi32(_tmp9, _tmpb);
+    __m128i _tmpr = _mm_unpackhi_epi32(_tmp9, _tmpb);
+    __m128i _tmps = _mm_unpacklo_epi32(_tmpc, _tmpe);
+    __m128i _tmpt = _mm_unpackhi_epi32(_tmpc, _tmpe);
+    __m128i _tmpu = _mm_unpacklo_epi32(_tmpd, _tmpf);
+    __m128i _tmpv = _mm_unpackhi_epi32(_tmpd, _tmpf);
 
-    _tmp0 = _mm256_unpacklo_epi64(_tmpg, _tmpk);
-    _tmp1 = _mm256_unpackhi_epi64(_tmpg, _tmpk);
-    _tmp2 = _mm256_unpacklo_epi64(_tmph, _tmpl);
-    _tmp3 = _mm256_unpackhi_epi64(_tmph, _tmpl);
-    _tmp4 = _mm256_unpacklo_epi64(_tmpi, _tmpm);
-    _tmp5 = _mm256_unpackhi_epi64(_tmpi, _tmpm);
-    _tmp6 = _mm256_unpacklo_epi64(_tmpj, _tmpn);
-    _tmp7 = _mm256_unpackhi_epi64(_tmpj, _tmpn);
-
-    _r0 = _mm256_permute2x128_si256(_tmp0, _tmp1, _MM_SHUFFLE(0, 2, 0, 0));
-    _r1 = _mm256_permute2x128_si256(_tmp2, _tmp3, _MM_SHUFFLE(0, 2, 0, 0));
-    _r2 = _mm256_permute2x128_si256(_tmp4, _tmp5, _MM_SHUFFLE(0, 2, 0, 0));
-    _r3 = _mm256_permute2x128_si256(_tmp6, _tmp7, _MM_SHUFFLE(0, 2, 0, 0));
-    _r4 = _mm256_permute2x128_si256(_tmp0, _tmp1, _MM_SHUFFLE(0, 3, 0, 1));
-    _r5 = _mm256_permute2x128_si256(_tmp2, _tmp3, _MM_SHUFFLE(0, 3, 0, 1));
-    _r6 = _mm256_permute2x128_si256(_tmp4, _tmp5, _MM_SHUFFLE(0, 3, 0, 1));
-    _r7 = _mm256_permute2x128_si256(_tmp6, _tmp7, _MM_SHUFFLE(0, 3, 0, 1));
+    _r0 = _mm_unpacklo_epi64(_tmpg, _tmpk);
+    _r1 = _mm_unpacklo_epi64(_tmpo, _tmps);
+    _r2 = _mm_unpackhi_epi64(_tmpg, _tmpk);
+    _r3 = _mm_unpackhi_epi64(_tmpo, _tmps);
+    _r4 = _mm_unpacklo_epi64(_tmph, _tmpl);
+    _r5 = _mm_unpacklo_epi64(_tmpp, _tmpt);
+    _r6 = _mm_unpackhi_epi64(_tmph, _tmpl);
+    _r7 = _mm_unpackhi_epi64(_tmpp, _tmpt);
+    _r8 = _mm_unpacklo_epi64(_tmpi, _tmpm);
+    _r9 = _mm_unpacklo_epi64(_tmpq, _tmpu);
+    _ra = _mm_unpackhi_epi64(_tmpi, _tmpm);
+    _rb = _mm_unpackhi_epi64(_tmpq, _tmpu);
+    _rc = _mm_unpacklo_epi64(_tmpj, _tmpn);
+    _rd = _mm_unpacklo_epi64(_tmpr, _tmpv);
+    _re = _mm_unpackhi_epi64(_tmpj, _tmpn);
+    _rf = _mm_unpackhi_epi64(_tmpr, _tmpv);
 }
 
 static NCNN_FORCEINLINE float _mm512_comp_reduce_add_ps(__m512 x)
@@ -985,6 +1291,19 @@ static NCNN_FORCEINLINE float _mm512_comp_reduce_max_ps(__m512 x)
     return _mm_cvtss_f32(x32);
 }
 
+static NCNN_FORCEINLINE __m128i float2int8_avx512(const __m512& _v0)
+{
+    // _MM_FROUND_TO_NEAREST_INT round to even
+    // simulate round to nearest via +/-0.5 with round to zero
+    __m512 _p5 = _mm512_set1_ps(0.5f);
+    __m512 _signmask = _mm512_castsi512_ps(_mm512_set1_epi32(1 << 31));
+    __m512 _sign = _mm512_and_ps(_v0, _signmask);
+    __m512 _v0_p5 = _mm512_or_ps(_p5, _sign);
+    __m512 _v0_adj = _mm512_add_ps(_v0, _v0_p5);
+    __m512i _v0_i = _mm512_cvttps_epi32(_v0_adj);
+    return _mm512_cvtepi32_epi8(_v0_i);
+}
+
 static NCNN_FORCEINLINE __m512 bfloat2float_avx512(const __m256i& v0)
 {
 #if __AVX512BF16__
@@ -1034,6 +1353,7 @@ static NCNN_FORCEINLINE __m512i float2bfloat_avx512(const __m512& v0, const __m5
 }
 
 #endif // __AVX512F__
+#endif // __AVX2__
 #endif // __AVX__
 #endif // __SSE2__
 
