@@ -41,10 +41,6 @@
 #include "option.h"
 #include "platform.h"
 
-#if NCNN_VULKAN
-#include <vulkan/vulkan.h>
-#endif // NCNN_VULKAN
-
 #if NCNN_PIXEL
 #if NCNN_PLATFORM_API
 #if __ANDROID_API__ >= 9
@@ -111,11 +107,15 @@ public:
 #if __ARM_NEON
     void fill(float32x4_t _v);
     void fill(uint16x4_t _v);
+#if !defined(_MSC_VER)
     void fill(int32x4_t _v);
+#endif
     void fill(int32x4_t _v0, int32x4_t _v1);
 #if __ARM_FEATURE_FP16_VECTOR_ARITHMETIC
+#if !defined(_MSC_VER)
     void fill(float16x4_t _v);
     void fill(float16x8_t _v);
+#endif
 #endif // __ARM_FEATURE_FP16_VECTOR_ARITHMETIC
 #endif // __ARM_NEON
 #if __SSE2__
@@ -963,6 +963,7 @@ NCNN_FORCEINLINE void Mat::fill(uint16x4_t _v)
     }
 }
 
+#if !defined(_MSC_VER)
 NCNN_FORCEINLINE void Mat::fill(int32x4_t _v)
 {
     int size = (int)total();
@@ -973,6 +974,7 @@ NCNN_FORCEINLINE void Mat::fill(int32x4_t _v)
         ptr += 4;
     }
 }
+#endif
 
 NCNN_FORCEINLINE void Mat::fill(int32x4_t _v0, int32x4_t _v1)
 {
@@ -986,6 +988,7 @@ NCNN_FORCEINLINE void Mat::fill(int32x4_t _v0, int32x4_t _v1)
     }
 }
 #if __ARM_FEATURE_FP16_VECTOR_ARITHMETIC
+#if !defined(_MSC_VER)
 NCNN_FORCEINLINE void Mat::fill(float16x4_t _v)
 {
     int size = (int)total();
@@ -1007,6 +1010,7 @@ NCNN_FORCEINLINE void Mat::fill(float16x8_t _v)
         ptr += 8;
     }
 }
+#endif
 #endif // __ARM_FEATURE_FP16_VECTOR_ARITHMETIC
 #endif // __ARM_NEON
 
