@@ -32,6 +32,7 @@
 #include "layer/batchnorm.h"
 #include "layer/bias.h"
 #include "layer/binaryop.h"
+#include "layer/celu.h"
 #include "layer/clip.h"
 #include "layer/concat.h"
 #include "layer/convolution.h"
@@ -51,6 +52,7 @@
 #include "layer/deconvolutiondepthwise3d.h"
 #include "layer/deformableconv2d.h"
 #include "layer/detectionoutput.h"
+#include "layer/diag.h"
 #include "layer/dropout.h"
 #include "layer/eltwise.h"
 #include "layer/elu.h"
@@ -794,7 +796,7 @@ int ModelWriter::save(const char* parampath, const char* binpath)
             continue;
         }
 
-        ncnn::Layer* layer_default = ncnn::create_layer(layer->typeindex);
+        ncnn::Layer* layer_default = ncnn::create_layer_cpu(layer->typeindex);
 
         ncnn::ParamDict pd;
         layer_default->load_param(pd);
@@ -835,6 +837,13 @@ int ModelWriter::save(const char* parampath, const char* binpath)
             fprintf_param_value(" 1=%d", with_scalar)
             fprintf_param_value(" 2=%e", b)
         }
+        else if (layer->type == "CELU")
+        {
+            ncnn::CELU* op = (ncnn::CELU*)layer;
+            ncnn::CELU* op_default = (ncnn::CELU*)layer_default;
+
+            fprintf_param_value(" 0=%e", alpha)
+        }
         else if (layer->type == "Clip")
         {
             ncnn::Clip* op = (ncnn::Clip*)layer;
@@ -888,18 +897,21 @@ int ModelWriter::save(const char* parampath, const char* binpath)
             }
             fprintf_param_value(" 19=%d", dynamic_weight)
 
-            fwrite_weight_tag_data(op->weight_data, bp);
-            fwrite_weight_data(op->bias_data, bp);
+            if (op->dynamic_weight == 0)
+            {
+                fwrite_weight_tag_data(op->weight_data, bp);
+                fwrite_weight_data(op->bias_data, bp);
 
 #if NCNN_INT8
-            // write int8_scale data
-            if (op->int8_scale_term)
-            {
-                fwrite_weight_data(op->weight_data_int8_scales, bp, 90, 100);
-                fwrite_weight_data(op->bottom_blob_int8_scales, bp, 0.001, 1);
-                fwrite_weight_data(op->top_blob_int8_scales, bp, 0.001, 1);
-            }
+                // write int8_scale data
+                if (op->int8_scale_term)
+                {
+                    fwrite_weight_data(op->weight_data_int8_scales, bp, 90, 100);
+                    fwrite_weight_data(op->bottom_blob_int8_scales, bp, 0.001, 1);
+                    fwrite_weight_data(op->top_blob_int8_scales, bp, 0.001, 1);
+                }
 #endif // NCNN_INT8
+            }
 
             if (shape_ready)
             {
@@ -931,9 +943,13 @@ int ModelWriter::save(const char* parampath, const char* binpath)
             {
                 if (!op->activation_params.empty()) fprintf_param_float_array(10, op->activation_params, pp);
             }
+            fprintf_param_value(" 19=%d", dynamic_weight)
 
-            fwrite_weight_tag_data(op->weight_data, bp);
-            fwrite_weight_data(op->bias_data, bp);
+            if (op->dynamic_weight == 0)
+            {
+                fwrite_weight_tag_data(op->weight_data, bp);
+                fwrite_weight_data(op->bias_data, bp);
+            }
 
             if (shape_ready)
             {
@@ -1040,32 +1056,35 @@ int ModelWriter::save(const char* parampath, const char* binpath)
             }
             fprintf_param_value(" 19=%d", dynamic_weight)
 
-            fwrite_weight_tag_data(op->weight_data, bp);
-            fwrite_weight_data(op->bias_data, bp);
+            if (op->dynamic_weight == 0)
+            {
+                fwrite_weight_tag_data(op->weight_data, bp);
+                fwrite_weight_data(op->bias_data, bp);
 
 #if NCNN_INT8
-            // write int8_scale data
-            if (op->int8_scale_term == 1 || op->int8_scale_term == 101)
-            {
-                op->bottom_blob_int8_scales.w = 1;
-            }
-            if (op->int8_scale_term == 2 || op->int8_scale_term == 102)
-            {
-                op->weight_data_int8_scales.w = 1;
-                op->bottom_blob_int8_scales.w = 1;
-            }
-            if (op->int8_scale_term > 100)
-            {
-                op->top_blob_int8_scales.w = 1;
-            }
+                // write int8_scale data
+                if (op->int8_scale_term == 1 || op->int8_scale_term == 101)
+                {
+                    op->bottom_blob_int8_scales.w = 1;
+                }
+                if (op->int8_scale_term == 2 || op->int8_scale_term == 102)
+                {
+                    op->weight_data_int8_scales.w = 1;
+                    op->bottom_blob_int8_scales.w = 1;
+                }
+                if (op->int8_scale_term > 100)
+                {
+                    op->top_blob_int8_scales.w = 1;
+                }
 
-            if (op->int8_scale_term)
-            {
-                fwrite_weight_data(op->weight_data_int8_scales, bp, 90, 100);
-                fwrite_weight_data(op->bottom_blob_int8_scales, bp, 0.001, 1);
-                fwrite_weight_data(op->top_blob_int8_scales, bp, 0.001, 1);
-            }
+                if (op->int8_scale_term)
+                {
+                    fwrite_weight_data(op->weight_data_int8_scales, bp, 90, 100);
+                    fwrite_weight_data(op->bottom_blob_int8_scales, bp, 0.001, 1);
+                    fwrite_weight_data(op->top_blob_int8_scales, bp, 0.001, 1);
+                }
 #endif // NCNN_INT8
+            }
 
             if (shape_ready)
             {
@@ -1098,9 +1117,13 @@ int ModelWriter::save(const char* parampath, const char* binpath)
             {
                 if (!op->activation_params.empty()) fprintf_param_float_array(10, op->activation_params, pp);
             }
+            fprintf_param_value(" 19=%d", dynamic_weight)
 
-            fwrite_weight_tag_data(op->weight_data, bp);
-            fwrite_weight_data(op->bias_data, bp);
+            if (op->dynamic_weight == 0)
+            {
+                fwrite_weight_tag_data(op->weight_data, bp);
+                fwrite_weight_data(op->bias_data, bp);
+            }
 
             if (shape_ready)
             {
@@ -1261,9 +1284,13 @@ int ModelWriter::save(const char* parampath, const char* binpath)
             {
                 if (!op->activation_params.empty()) fprintf_param_float_array(10, op->activation_params, pp);
             }
+            fprintf_param_value(" 28=%d", dynamic_weight)
 
-            fwrite_weight_tag_data(op->weight_data, bp);
-            fwrite_weight_data(op->bias_data, bp);
+            if (op->dynamic_weight == 0)
+            {
+                fwrite_weight_tag_data(op->weight_data, bp);
+                fwrite_weight_data(op->bias_data, bp);
+            }
 
             if (shape_ready)
             {
@@ -1296,9 +1323,13 @@ int ModelWriter::save(const char* parampath, const char* binpath)
             {
                 if (!op->activation_params.empty()) fprintf_param_float_array(10, op->activation_params, pp);
             }
+            fprintf_param_value(" 28=%d", dynamic_weight)
 
-            fwrite_weight_tag_data(op->weight_data, bp);
-            fwrite_weight_data(op->bias_data, bp);
+            if (op->dynamic_weight == 0)
+            {
+                fwrite_weight_tag_data(op->weight_data, bp);
+                fwrite_weight_data(op->bias_data, bp);
+            }
 
             if (shape_ready)
             {
@@ -1418,9 +1449,13 @@ int ModelWriter::save(const char* parampath, const char* binpath)
             {
                 if (!op->activation_params.empty()) fprintf_param_float_array(10, op->activation_params, pp);
             }
+            fprintf_param_value(" 28=%d", dynamic_weight)
 
-            fwrite_weight_tag_data(op->weight_data, bp);
-            fwrite_weight_data(op->bias_data, bp);
+            if (op->dynamic_weight == 0)
+            {
+                fwrite_weight_tag_data(op->weight_data, bp);
+                fwrite_weight_data(op->bias_data, bp);
+            }
 
             if (shape_ready)
             {
@@ -1454,9 +1489,13 @@ int ModelWriter::save(const char* parampath, const char* binpath)
             {
                 if (!op->activation_params.empty()) fprintf_param_float_array(10, op->activation_params, pp);
             }
+            fprintf_param_value(" 28=%d", dynamic_weight)
 
-            fwrite_weight_tag_data(op->weight_data, bp);
-            fwrite_weight_data(op->bias_data, bp);
+            if (op->dynamic_weight == 0)
+            {
+                fwrite_weight_tag_data(op->weight_data, bp);
+                fwrite_weight_data(op->bias_data, bp);
+            }
 
             if (shape_ready)
             {
@@ -1597,6 +1636,13 @@ int ModelWriter::save(const char* parampath, const char* binpath)
             fprintf_param_value(" 7=%e", variances[2])
             fprintf_param_value(" 8=%e", variances[3])
         }
+        else if (layer->type == "Diag")
+        {
+            ncnn::Diag* op = (ncnn::Diag*)layer;
+            ncnn::Diag* op_default = (ncnn::Diag*)layer_default;
+
+            fprintf_param_value(" 0=%d", diagonal)
+        }
         else if (layer->type == "Dropout")
         {
             ncnn::Dropout* op = (ncnn::Dropout*)layer;
@@ -1718,6 +1764,19 @@ int ModelWriter::save(const char* parampath, const char* binpath)
             fprintf_param_value(" 20=%d", constant_TILE_M)
             fprintf_param_value(" 21=%d", constant_TILE_N)
             fprintf_param_value(" 22=%d", constant_TILE_K)
+
+            if (op->constantA == 1)
+            {
+                fwrite_weight_tag_data(op->A_data, bp);
+            }
+            if (op->constantB == 1)
+            {
+                fwrite_weight_tag_data(op->B_data, bp);
+            }
+            if (op->constantC == 1 && op->constant_broadcast_type_C != -1)
+            {
+                fwrite_weight_tag_data(op->C_data, bp);
+            }
         }
         else if (layer->type == "GLU")
         {
@@ -1734,6 +1793,7 @@ int ModelWriter::save(const char* parampath, const char* binpath)
             fprintf_param_value(" 0=%d", sample_type)
             fprintf_param_value(" 1=%d", padding_mode)
             fprintf_param_value(" 2=%d", align_corner)
+            fprintf_param_value(" 3=%d", permute_fusion)
         }
         else if (layer->type == "GroupNorm")
         {
@@ -1756,10 +1816,20 @@ int ModelWriter::save(const char* parampath, const char* binpath)
             fprintf_param_value(" 0=%d", num_output)
             fprintf_param_value(" 1=%d", weight_data_size)
             fprintf_param_value(" 2=%d", direction)
+            fprintf_param_value(" 8=%d", int8_scale_term)
 
             fwrite_weight_tag_data(op->weight_xc_data, bp);
             fwrite_weight_tag_data(op->bias_c_data, bp);
             fwrite_weight_tag_data(op->weight_hc_data, bp);
+
+#if NCNN_INT8
+            // write int8_scale data
+            if (op->int8_scale_term)
+            {
+                fwrite_weight_data(op->weight_xc_data_int8_scales, bp, 90, 100);
+                fwrite_weight_data(op->weight_hc_data_int8_scales, bp, 90, 100);
+            }
+#endif // NCNN_INT8
         }
         else if (layer->type == "HardSigmoid")
         {
@@ -1888,6 +1958,7 @@ int ModelWriter::save(const char* parampath, const char* binpath)
             fprintf_param_value(" 1=%d", weight_data_size)
             fprintf_param_value(" 2=%d", direction)
             fprintf_param_value(" 3=%d", hidden_size)
+            fprintf_param_value(" 8=%d", int8_scale_term)
 
             fwrite_weight_tag_data(op->weight_xc_data, bp);
             fwrite_weight_tag_data(op->bias_c_data, bp);
@@ -1897,6 +1968,15 @@ int ModelWriter::save(const char* parampath, const char* binpath)
             {
                 fwrite_weight_tag_data(op->weight_hr_data, bp);
             }
+
+#if NCNN_INT8
+            // write int8_scale data
+            if (op->int8_scale_term)
+            {
+                fwrite_weight_data(op->weight_xc_data_int8_scales, bp, 90, 100);
+                fwrite_weight_data(op->weight_hc_data_int8_scales, bp, 90, 100);
+            }
+#endif // NCNN_INT8
         }
         else if (layer->type == "MatMul")
         {
@@ -1922,10 +2002,12 @@ int ModelWriter::save(const char* parampath, const char* binpath)
             ncnn::MultiHeadAttention* op_default = (ncnn::MultiHeadAttention*)layer_default;
 
             fprintf_param_value(" 0=%d", embed_dim)
-            fprintf_param_value(" 1=%d", num_head)
+            fprintf_param_value(" 1=%d", num_heads)
             fprintf_param_value(" 2=%d", weight_data_size)
             fprintf_param_value(" 3=%d", kdim)
             fprintf_param_value(" 4=%d", vdim)
+            fprintf_param_value(" 5=%d", attn_mask)
+            fprintf_param_value(" 6=%e", scale)
 
             fwrite_weight_tag_data(op->q_weight_data, bp);
             fwrite_weight_data(op->q_bias_data, bp);
@@ -2228,10 +2310,20 @@ int ModelWriter::save(const char* parampath, const char* binpath)
             fprintf_param_value(" 0=%d", num_output)
             fprintf_param_value(" 1=%d", weight_data_size)
             fprintf_param_value(" 2=%d", direction)
+            fprintf_param_value(" 8=%d", int8_scale_term)
 
             fwrite_weight_tag_data(op->weight_xc_data, bp);
             fwrite_weight_tag_data(op->bias_c_data, bp);
             fwrite_weight_tag_data(op->weight_hc_data, bp);
+
+#if NCNN_INT8
+            // write int8_scale data
+            if (op->int8_scale_term)
+            {
+                fwrite_weight_data(op->weight_xc_data_int8_scales, bp, 90, 100);
+                fwrite_weight_data(op->weight_hc_data_int8_scales, bp, 90, 100);
+            }
+#endif // NCNN_INT8
         }
         else if (layer->type == "ROIAlign")
         {
@@ -2307,7 +2399,7 @@ int ModelWriter::save(const char* parampath, const char* binpath)
             fprintf_param_value(" 11=%d", squeeze_d)
             fprintf_param_value(" 2=%d", squeeze_c)
             {
-                if (!op->axes.empty()) fprintf_param_int_array(0, op->axes, pp);
+                if (!op->axes.empty()) fprintf_param_int_array(3, op->axes, pp);
             }
         }
         else if (layer->type == "Threshold")
